#!/usr/bin/perl

# Copyright INRA/CNRS
#
# sebastien.carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME
 
	pasa_annotation_cleaner.pl
	
=head1 SYNOPSIS

	 pasa_annotation_cleaner.pl --gff3_file=<file> --outfile=<file>

=head1 DESCRIPTION

	Detect if several mRNA for a same gene are predicted on different strand. Keep mRNA predicted on the same strand than the longest mRNA, remove the others mRNA from the annotation.

=head1 AUTHORS
 
 Marion Verdenaud	marion.verdenaud@toulouse.inra.fr
 Sebastien Carrere  sebastien.carrere@toulouse.inra.fr
 Jerome Gouzy	    Jerome.Gouzy@toulouse.inra.fr

=cut

my ($dirprg, $nameprg);
BEGIN
{

   ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    
    $dirprg = '.' if (! (defined $dirprg) || ($dirprg eq ''));
    
    if (-d "$dirprg/lib")
    {
        unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }
}

use strict;
use warnings;
use ParamParser;
use IO::File;


my $O_PARAM = New ParamParser('GETOPTLONG',("gff3_file=s","outfile=s"));

MAIN:
{
	$O_PARAM->SetUsage(my $usage = sub {&Usage();});
	
	$O_PARAM->AssertDefined("gff3_file");
	$O_PARAM->AssertFileExists("gff3_file");
	$O_PARAM->AssertDefined("outfile");

	my $gff3_file_handle= new IO::File($O_PARAM->Get("gff3_file"));
	my $outfile=new IO::File(">".$O_PARAM->Get("outfile"));

	my %h_ids_delete;
	my %h_gene_change;
	my %h_genes;
	while (my $line=<$gff3_file_handle>)
	{
		#on recupere les ids des mRNA, le parent, le brin, la longueur
		if ($line =~ /mRNA/)
		{
			my @a_line=split("\t",$line);
			my $desc=$a_line[8];
			
			my %h_mrna;
			my ($parent) = $desc =~ /Parent=([^;\s]+)/;
			($h_mrna{'parent'})=$parent;
			($h_mrna{'id'}) = $desc =~ /ID=([^;\s]+)/;
			($h_mrna{'length'}) = $desc =~ /length=([\d]+)/;
			($h_mrna{'brin'})=$a_line[6];
			
			
			my @a_mrnas;
			if (defined $h_genes{$parent})
			{
				@a_mrnas=@{$h_genes{$parent}};
			}
			push(@a_mrnas,\%h_mrna);
			$h_genes{$parent}=\@a_mrnas;
		}
	}

	foreach my $gene (keys %h_genes)
	{
		my @a_mrna=@{$h_genes{$gene}};
		my $brin="";
		my $diff=0;

		#on regarde si les mRNA du meme gene sont sur des brins differents
		foreach my $rh_mrna (@a_mrna)
		{
			my $brin_mrna=$rh_mrna->{'brin'};
			if ($brin eq "")
			{
				$brin=$brin_mrna;
			}
			else
			{
				if ($brin ne $brin_mrna)
				{
					$diff=1;
					last;
				}
			}
		}

		#on a trouve plusieurs brin pour les mRNA du meme gene
		if ($diff == 1)
		{
			my $max_length=0;
			my $max_length_brin;
			#on regarde sur quel brin est le mRNA le plus long
			foreach my $rh_mrna (@a_mrna)
			{
				my $length_mrna=$rh_mrna->{'length'};
				my $brin_mrna=$rh_mrna->{'brin'};
				if ($max_length eq "" || $length_mrna>$max_length)
				{
					$max_length=$length_mrna;
					$max_length_brin=$brin_mrna;
				}

			}

			#on ne garde que les mRNAs sur ce brin
			foreach my $rh_mrna (@a_mrna)
			{
				my $brin_mrna=$rh_mrna->{'brin'};
				if ($max_length_brin ne $brin_mrna)
				{
					$h_ids_delete{$rh_mrna->{'id'}}='';
					$h_gene_change{$rh_mrna->{'parent'}}=$max_length_brin if (! $h_gene_change{$rh_mrna->{'parent'}});
				}
			}
		}
	}

	$gff3_file_handle->seek(0,'SEEK_SET');
	while (my $line=<$gff3_file_handle>)
	{
		my ($id) = $line =~ /ID=([^;\s]+)/;
		my ($parent) = $line =~ /Parent=([^;\s]+)/;
		$parent = "" if (! defined $parent);
		if (defined $h_gene_change{$id})
		{
			my @a_line=split("\t",$line);
			$a_line[6]=$h_gene_change{$id};
			print $outfile join("\t",@a_line);
		}
		elsif ( ! (defined $h_ids_delete{$id} || defined $h_ids_delete{$parent}))
		{
			print $outfile $line;
		}

	}
	$gff3_file_handle->close;
	$outfile->close;

}

sub Usage
{
 print STDOUT <<END;

Usage:

     $0 --gff3_file=<file> --outfile=<file>

END


}