#!/usr/bin/perl

# Copyright INRA/CNRS
#
# sebastien.carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

 Compendium.pl - 

=head1 SYNOPSIS

 Compendium.pl --genome_file <file> --transcripts_file <file> --cfg_file <file>  --work_repository <dir> [--full_length_file <file>]  [--paired_ends_file <file>] [--quality_file] [--step <string>] [--cpus <integer|filename>] [--tgicl_restart <integer>] [--annotation_file <filename>] [update_annotation_pasa_only]

=head1 DESCRIPTION

 Compendium.pl :
	- Launch Velvet_Oases in order to assemble into contigs and transcripts Solexa short reads
	- Launch Pasa pipeline, in order to align the transcripts on the genome.
	- Launch TGICL++ in order to cluster the transcripts that don't match on the genome.
	- Launch FrameDP on the contigs output of TGICL to identify chimera or splicing alternative.

    Mandatory
        --genome_file                  : format fasta file of the genome
        --transcripts_file             : format fasta file with all the transcripts sequences
        --cfg_file                     : the configuration file, in wich parameters of Pasa and TGICL are modifiable.
        --work_repository              : the path of the work directory

    Optional
        --quality_file                 : quality file of the transcripts
        --full_length_file             : file with the identifiers of the transcripts known as full lengths (this ids have to be present in the transcripts file !)
        --paired_ends_file             : file with the identifiers of couple of transcripts known as paired ends (the ids of the couple have to be on the same line)
                           eg : AL367179	AL367180
                                MtRH15N17A1	MtRH15N17N1
        --annotation_file              : format gff3 file annotation of the genome
        --update_annotation_pasa_only  : if defined, the script will only update the annotation and compare it with PASA annotation
        --step                         : run particular step of the pipeline, allowed values : $allowed
        --cpus                         : either a number of cpus or a file containing the list of pvm nodes
        --tgicl_restart                : if tgicl is use with the simplification_step (see configuration file), you can define a restart step


=head1 AUTHORS
 
 Marion Verdenaud	marion.verdenaud@toulouse.inra.fr
 Sebastien Carrere  sebastien.carrere@toulouse.inra.fr
 Jerome Gouzy	    Jerome.Gouzy@toulouse.inra.fr

=cut

use strict;
use warnings;
my ($dirprg, $nameprg);

BEGIN
{
    ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    $dirprg = '.' if (!defined $dirprg);
    $ENV{COMPENDIUM} = "$dirprg/.." if (!defined($ENV{COMPENDIUM}));
    unshift(@INC, "$dirprg/lib");
}

our $VERSION = do {my @r = (q$Rev: 326 $ =~ /\d+/g); $r[0]};

use ParamParser;
use IO::File;
use Cwd qw/ abs_path /;
use Logger;
use BerkeleyDB;
use File::Basename;

$ENV{LANG}="C";

my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
$mon++;
$mon = $mon < 10 ? $mon = "0" . $mon : $mon;
my $DATE = (1900 + $year) . $mon . $mday;

my $O_PARAM = New ParamParser(
                              "GETOPTLONG",
                              (
                               "genome_file=s",                       "short_reads_librairies=s",
                               "short_reads_type=s",                  "short_reads_format=s",
                               "assemble_short_reads_lib_one_by_one", "long_reads_librairies=s",
                               "quality_file=s",                      "full_length_file=s",
                               "paired_ends_file=s",                  "cfg_file=s",
                               "work_repository=s",                   "step=s",
                               "cpus=s",                              "tgicl_restart=s",
                               "annotation_file=s",                   "update_annotation_pasa_only"
                                )
                                );

my @A_STEPS = ('VELVET_OASES', 'PASA', 'TGICL', 'SOAP', 'FRAMEDP');

#a mettre dans le fichier de conf ?
my @A_VELVET_FORMAT = ('fasta', 'fastq', 'fasta.gz', 'fastq.gz', 'sam', 'bam', 'eland', 'gerald');
my @A_VELVET_TYPE = ('short', 'shortPaired', 'short2', 'shortPaired2', 'long', 'longPaired');

my $ln   = "ln";
my $gzip = "gzip";
my $cat  = "cat";

MAIN:
{
    $O_PARAM->SetUsage(my $usage = sub {&Usage();});

    my $rh_inputs = &CheckInputs();

    my $genome          = $rh_inputs->{'genome'};
    my $genome_name     = $rh_inputs->{'genome_name'};
    my $paired_ends     = $rh_inputs->{'paired_ends'};
    my $annotation_file = $rh_inputs->{'annotation_file'};
    my $full_length     = $rh_inputs->{'full_length'};

    my $cpus       = $O_PARAM->Get('cpus');
    my $repository = abs_path($O_PARAM->Get('work_repository'));

    my $stat_path = "$repository/pipeline.stat";

    $O_PARAM->Update($O_PARAM->Get('cfg_file'),             'A');
    $O_PARAM->Update("$dirprg/cfg/compendium_programs.cfg", 'A');

    $ln   = $O_PARAM->Get('unix_ln')   if ($O_PARAM->IsDefined('unix_ln'));
    $gzip = $O_PARAM->Get('unix_gzip') if ($O_PARAM->IsDefined('unix_gzip'));
    $cat  = $O_PARAM->Get('unix_cat')  if ($O_PARAM->IsDefined('unix_cat'));

    my $o_logger_log = New Logger(">>" . $repository . '/' . $O_PARAM->Get('log_pipeline') . $O_PARAM->Get('log_ext'),
                                  $O_PARAM->Get('log_level'),
                                  $O_PARAM->Get('log_options'));

    my $o_logger_error =
      New Logger(">>" . $repository . '/' . $O_PARAM->Get('error_pipeline') . $O_PARAM->Get('error_ext'),
                 $O_PARAM->Get('error_level'),
                 $O_PARAM->Get('error_options'));

    my $ra_long_reads    = $rh_inputs->{'a_long_lib'};
    my $transcripts      = "";
    my $transcripts_name = "";
    if (scalar(@$ra_long_reads) > 1)
    {
        $transcripts      = "$repository/compendium_long_reads_input.fasta";
        $transcripts_name = "long_reads_input.fasta";
        my $cmd_cat = "$cat " . join(" ", @$ra_long_reads) . "> $transcripts";
        system($cmd_cat);
    }
    elsif (scalar(@$ra_long_reads) == 1)
    {
        $transcripts = @$ra_long_reads[0];
        ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
    }

    my $ra_quality = $rh_inputs->{'quality'};
    my $quality;
    if (scalar(@$ra_quality) > 1)
    {
        $quality = "$repository/compendium_long_reads_input.qual";
        my $cmd_cat = "$cat " . join(" ", @$ra_quality) . "> $quality";
        system($cmd_cat);
    }
    elsif (scalar(@$ra_quality) == 1)
    {
        $quality = @$ra_quality[0];
    }

    my $velvet_oases_results_use = $O_PARAM->Get('velvet_oases_results_use');
    if ($velvet_oases_results_use ne "contigs.fa")
    {
        $velvet_oases_results_use = "transcripts.fa";
    }

    if ($O_PARAM->IsDefined('step'))
    {
        my $step = $O_PARAM->Get('step');
        if ($step =~ /PASA/)
        {
            $O_PARAM->AssertDefined('genome_file');
            $O_PARAM->AssertFileExists('genome_file');
        }
        if ($step =~ /SOAP/)
        {
            $O_PARAM->AssertDefined('short_reads_librairies');
        }

        if ($step =~ /VELVET_OASES/)
        {
            $O_PARAM->AssertDefined('short_reads_librairies');
            $O_PARAM->AssertDefined('short_reads_type');
            $O_PARAM->AssertDefined('short_reads_format');

            $o_logger_error->Trace("\n\n##########\n# VELVET  #\n##########\n\n", "", "ERROR");
            mkdir("$repository/0_VELVET_OASES");
            chdir("$repository/0_VELVET_OASES");

            &Velvet_Oases($repository,
                          $rh_inputs->{'a_short_lib'},
                          $rh_inputs->{'a_short_type'},
                          $rh_inputs->{'a_short_format'},
                          $cpus, $velvet_oases_results_use, $o_logger_log, $o_logger_error);
            if (!-e "$repository/0_VELVET_OASES/$velvet_oases_results_use")
            {
                $o_logger_error->Trace(
                    "##  VELVET/OASES ERROR : the file $repository/0_VELVET_OASES/$velvet_oases_results_use doesn't exist.",
                    "", "ERROR"
                    );
                exit();
            }
            $transcripts =
              &AfterOases($repository, $ra_long_reads, $velvet_oases_results_use, $o_logger_log, $o_logger_error);
            ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
        }

        #une etape velvet a deja ete realisee mais pas a ce tour, on recupere quand meme les infos.
        elsif (-r "$repository/0_VELVET_OASES")
        {
            if (-e "$repository/0_VELVET_OASES/$velvet_oases_results_use")
            {
                if (-e "$repository/0_VELVET_OASES/AFTER_OASES.$velvet_oases_results_use+long_reads.fasta")
                {
                    $transcripts = "$repository/0_VELVET_OASES/AFTER_OASES.$velvet_oases_results_use+long_reads.fasta";
                    ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
                }
                else
                {
                    $transcripts =
                      &AfterOases($repository, $ra_long_reads, $velvet_oases_results_use, $o_logger_log,
                                  $o_logger_error);
                    ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
                }
            }
            else
            {
                $o_logger_error->Trace(
                    "##  VELVET/OASES ERROR : the file $repository/0_VELVET_OASES/$velvet_oases_results_use doesn't exist.",
                    "", "ERROR"
                    );
                exit();
            }
        }
        else
        {
            $O_PARAM->AssertDefined('long_reads_librairies');
        }

        my $PASA_mRNA = "";
        if ($step =~ /PASA/)
        {
            if (($step =~ /VELVET_OASES/) && (!-e "$repository/0_VELVET_OASES/$velvet_oases_results_use"))
            {
                $o_logger_error->Trace(
                    "##  VELVET/OASES ERROR : the file $repository/0_VELVET_OASES/$velvet_oases_results_use doesn't exist.",
                    "", "ERROR"
                    );
                exit();
            }
            if (!$O_PARAM->IsDefined('update_annotation_pasa_only'))
            {
                &Pasa($repository, $transcripts, $transcripts_name, $genome, $genome_name, $full_length, $o_logger_log,
                      $o_logger_error);
                if ((!-e "$repository/1_PASA/validated_transcripts.gff3"))
                {
                    $o_logger_error->Trace(
                               "##  PASA ERROR : the file $repository/1_PASA/validated_transcripts.gff3 doesn't exist.",
                               "", "ERROR");
                    exit();
                }
                &PasaStat($repository, $transcripts_name, $paired_ends, $stat_path, $o_logger_error);
            }
            if ($O_PARAM->IsDefined('annotation_file'))
            {
                &PasaAnnotation($repository,      $genome_name,  $transcripts_name,
                                $annotation_file, $o_logger_log, $o_logger_error);
                &AnalysePasaAnnotationOutput($repository, $genome, $annotation_file, $o_logger_log, $o_logger_error);
            }
            ($transcripts, $paired_ends, $quality, $PASA_mRNA) =
              &AfterPasa($repository, $transcripts_name, $paired_ends, $full_length, $quality, $o_logger_log,
                         $o_logger_error);
            ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
        }

        elsif (-r "$repository/1_PASA")
        {
            ($transcripts, $paired_ends, $quality, $PASA_mRNA) =
              &AfterPasa($repository, $transcripts_name, $paired_ends, $full_length, $quality, $o_logger_log,
                         $o_logger_error);
            ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
        }

        if ($step =~ /TGICL/)
        {

            #test si etape PASA s'est bien passee ?
            if (($step =~ /PASA/) && (!-e "$repository/1_PASA/validated_transcripts.gff3"))
            {
                $o_logger_error->Trace(
                               "##  PASA ERROR : the file $repository/1_PASA/validated_transcripts.gff3 doesn't exist.",
                               "", "ERROR");
                exit();
            }

            #test si etape VELVET OASES c'ets bien passee ?
            if (($step =~ /VELVET_OASES/) && (!-e "$repository/0_VELVET_OASES/$velvet_oases_results_use"))
            {
                $o_logger_error->Trace(
                    "##  VELVET/OASES ERROR : the file $repository/0_VELVET_OASES/$velvet_oases_results_use doesn't exist.",
                    "", "ERROR"
                    );
                exit();
            }
            else
            {
                &Tgicl($repository, $transcripts, $transcripts_name, $paired_ends, $quality, $O_PARAM->Get('cpus'),
                       $o_logger_log, $o_logger_error);
                if ((!-e "$repository/2_TGICL/tgicl_$transcripts_name.tgicl++.consensus.fasta"))
                {
                    $o_logger_error->Trace(
                                      "##  TGICL ERROR : the file $repository/2_TGICL/$transcripts_name doesn't exist.",
                                      "", "ERROR");
                    exit();
                }
                &TgiclStat($repository, $transcripts, $transcripts_name, $stat_path, $o_logger_error);
                $transcripts = "$repository/2_TGICL/tgicl_$transcripts_name.tgicl++.consensus.fasta";
                ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
            }
        }
        elsif (-r "$repository/2_TGICL")
        {
            $transcripts = "$repository/2_TGICL/tgicl_$transcripts_name.tgicl++.consensus.fasta";
            ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
        }

        my $transcripts_name_soap = $transcripts_name;
        my $transcripts_soap      = $transcripts;
        if ($step =~ /SOAP/)
        {
            mkdir("$repository/3_SOAP");
            chdir("$repository/3_SOAP");

            if (($step =~ /TGICL/) && (!-e "$repository/2_TGICL/$transcripts_name"))
            {
                $o_logger_error->Trace(
                                      "##  TGICL ERROR : the file $repository/2_TGICL/$transcripts_name doesn't exist.",
                                      "", "ERROR");
                exit();
            }
            if (($step =~ /PASA/) && (!-e "$repository/1_PASA/validated_transcripts.gff3"))
            {
                $o_logger_error->Trace(
                               "##  PASA ERROR : the file $repository/1_PASA/validated_transcripts.gff3 doesn't exist.",
                               "", "ERROR");
                exit();
            }

            #test si etape VELVET OASES c'est bien passee ?
            if (($step =~ /VELVET_OASES/) && (!-e "$repository/0_VELVET_OASES/$velvet_oases_results_use"))
            {
                $o_logger_error->Trace(
                    "##  VELVET/OASES ERROR : the file $repository/0_VELVET_OASES/$velvet_oases_results_use doesn't exist.",
                    "", "ERROR"
                    );
                exit();
            }

            #on cherche a genere le fichier des transcrits : PASA mRNAS + clusters TGICL, ou l'un ou l'autre
            if (-r "$repository/1_PASA")
            {
                if (-r "$repository/2_TGICL")
                {
                    system("$cat $PASA_mRNA $transcripts > $repository/3_SOAP/PASA_mRNAs+TGICLclusters.fasta");
                    $transcripts_name_soap = "PASA_mRNAs+TGICLclusters.fasta";
                }
                elsif (-r "$repository/0_VELVET_OASES")
                {
                    system(
                        "$cat $PASA_mRNA $transcripts > $repository/3_SOAP/PASA_mRNAs+OASES$velvet_oases_results_use.fasta"
                        );
                    $transcripts_name_soap = "PASA_mRNAs+OASES$velvet_oases_results_use.fasta";
                }
                else
                {
                    system("$ln -s $PASA_mRNA");
                    ($transcripts_name_soap) = $PASA_mRNA =~ /.+\/(.+)/;
                }
                $transcripts_soap = "$repository/3_SOAP/$transcripts_name_soap";
            }
            system("$ln -s $transcripts_soap") if (!-e "$repository/3_SOAP/$transcripts_name_soap");
            $transcripts_soap = "$repository/3_SOAP/$transcripts_name_soap";
            &Soap($repository, $transcripts_soap, $rh_inputs->{'a_short_lib'}, $cpus, $o_logger_log, $o_logger_error);
        }

        if ($step =~ /FRAMEDP/)
        {
            mkdir("$repository/4_FRAMEDP");
            chdir("$repository/4_FRAMEDP");

            if (($step =~ /TGICL/) && (!-e "$repository/2_TGICL/$transcripts_name"))
            {
                $o_logger_error->Trace(
                                      "##  TGICL ERROR : the file $repository/2_TGICL/$transcripts_name doesn't exist.",
                                      "", "ERROR");
                exit();
            }
            if (($step =~ /PASA/) && (!-e "$repository/1_PASA/validated_transcripts.gff3"))
            {
                $o_logger_error->Trace(
                               "##  PASA ERROR : the file $repository/1_PASA/validated_transcripts.gff3 doesn't exist.",
                               "", "ERROR");
                exit();
            }

            #test si etape VELVET OASES c'ets bien passee ?
            if (($step =~ /VELVET_OASES/) && (!-e "$repository/0_VELVET_OASES/$velvet_oases_results_use"))
            {
                $o_logger_error->Trace(
                    "##  VELVET/OASES ERROR : the file $repository/0_VELVET_OASES/$velvet_oases_results_use doesn't exist.",
                    "", "ERROR"
                    );
                exit();
            }
            if (-r "$repository/2_TGICL")
            {
                if (!-e "$repository/2_TGICL/$transcripts_name")
                {
                    $o_logger_error->Trace(
                                      "##  TGICL ERROR : the file $repository/2_TGICL/$transcripts_name doesn't exist.",
                                      "", "ERROR");
                    exit();
                }
                $transcripts = &AfterTgicl($repository, $transcripts_name, $o_logger_log, $o_logger_error);
                ($transcripts_name) = $transcripts =~ /.+\/(.+)/;
            }

            &FrameDP($repository, $transcripts, $o_logger_log, $o_logger_error);
            &FrameDPStat($repository, $stat_path);
            &FrameDPBuildCorrectedClustersDB($repository, $o_logger_log, $o_logger_error);
            &FrameDPTarFrameDPictures($repository, $o_logger_log, $o_logger_error);
        }

#on va analyser les fichiers report genere (si on a une etape Velvet/Oases de definie), pour generer les fichier report .bios
        &ConvertReportToBios($repository, $velvet_oases_results_use, $o_logger_log, $o_logger_error);

    }
}

sub ConvertReportToBios
{
    my ($repository, $velvet_oases_results_use, $o_logger_log, $o_logger_error) = @_;

#on va analyser les fichiers report genere (si on a une etape Velvet/Oases de definie), pour generer les fichier report .bios
#si une etape velvet/oases est ok
    if (-r "$repository/0_VELVET_OASES")
    {
        my %h_ids_velvet;
        if ((-e "$repository/0_VELVET_OASES/$velvet_oases_results_use"))
        {
            my @headers_ids = `grep ">" $repository/0_VELVET_OASES/$velvet_oases_results_use | cut -f2,2 -d '>'`;
            chomp @headers_ids;

            foreach my $ids (@headers_ids)
            {
                $h_ids_velvet{$ids} = '';
            }
        }
        else
        {
            $o_logger_error->Trace(
                "##  VELVET/OASES ERROR : the file $repository/0_VELVET_OASES/$velvet_oases_results_use doesn't exist.",
                "", "ERROR"
                );
            exit();
        }

        if ((-r "$repository/1_PASA"))
        {
            my @a_files = `ls $repository/1_PASA/*.report`;
            chomp @a_files;
            if (scalar(@a_files) == 0)
            {
                $o_logger_error->Trace(
                                 "##  PASA ERROR : No report file found in $repository/1_PASA does anything go wrong ?",
                                 "", "ERROR");
            }
            else
            {
                foreach my $file (@a_files)
                {
                    &GenerateBIOSreport($file, \%h_ids_velvet);
                }
            }
        }
        if ((-r "$repository/2_TGICL"))
        {
            my @a_files = `ls $repository/2_TGICL/*.report`;
            chomp @a_files;
            if (scalar(@a_files) == 0)
            {
                $o_logger_error->Trace(
                               "##  TGICL ERROR : No report file found in $repository/2_TGICL does anything go wrong ?",
                               "", "ERROR");
            }
            else
            {
                foreach my $file (@a_files)
                {
                    &GenerateBIOSreport($file, \%h_ids_velvet);
                }
            }
        }
    }

}

sub GenerateBIOSreport
{
    my ($file, $rh_ids_velvet) = @_;

    my $in  = new IO::File("$file")       || Carp::croak("ERROR : Can't open the file $file");
    my $out = new IO::File(">$file.bios") || Carp::croak("ERROR : Can't open the file $file.bios");
    my $header = "";
    while (my $line = <$in>)
    {
        chomp $line;
        if ($line =~ /^>(\S+)/)
        {
            $header = $line;
        }
        else
        {
            my @a_line = split(" ", $line);
            my @a_bios_ids;
            foreach my $id (@a_line)
            {
                if (!defined $rh_ids_velvet->{$id} && ($id ne ""))
                {
                    push(@a_bios_ids, $id);
                }
            }
            if (scalar @a_bios_ids > 0)
            {
                print $out $header . "\n";
                print $out join(" ", @a_bios_ids) . "\n";
            }
        }
    }
    $in->close();
    $out->close();

}

sub CheckInputs
{
    $O_PARAM->AssertDefined('cfg_file');
    $O_PARAM->AssertFileExists('cfg_file');
    $O_PARAM->AssertDefined('work_repository');
    $O_PARAM->AssertDirExists('work_repository');
    if ($O_PARAM->IsDefined('cpus'))
    {
        my $cpu = $O_PARAM->Get('cpus');
        if (-e $cpu)
        {
            $O_PARAM->Set('cpus', abs_path($cpu));
        }
    }
    $O_PARAM->SetUnlessDefined('cpus', 1);

    my $genome = abs_path($O_PARAM->Get('genome_file'));
    my ($genome_name) = $genome =~ /.+\/(.+)/;

    if ($O_PARAM->IsDefined('update_annotation_pasa_only'))
    {
        $O_PARAM->Set('step', 'PASA');
    }
    if ($O_PARAM->IsDefined('step'))
    {
        my @a_steps = split(",", $O_PARAM->Get('step'));
        foreach my $step (@a_steps)
        {
            &AllowedValue($step, \@A_STEPS, 'step');
        }
    }
    else
    {
        my $all_step = join(",", @A_STEPS);
        $O_PARAM->Set('step', $all_step);
    }

    my $paired_ends;
    my $annotation_file;
    if ($O_PARAM->IsDefined('paired_ends_file'))
    {
        $O_PARAM->AssertFileExists('paired_ends_file');
        $paired_ends = abs_path($O_PARAM->Get('paired_ends_file'));
    }
    my $full_length;
    if ($O_PARAM->IsDefined('full_length_file'))
    {
        $O_PARAM->AssertFileExists('full_length_file');
        $full_length = abs_path($O_PARAM->Get('full_length_file'));
    }
    if ($O_PARAM->IsDefined('tgicl_restart'))
    {
        $O_PARAM->AssertInteger('tgicl_restart');
        if ($O_PARAM->Get('tgicl_restart') < 0)
        {
            print STDERR "The tgicl restart step has to be a positive integer !!";
            exit;
        }
    }
    if ($O_PARAM->IsDefined('annotation_file'))
    {
        $O_PARAM->AssertFileExists('annotation_file');
        $annotation_file = abs_path($O_PARAM->Get('annotation_file'));
    }

    my @a_long_lib;
    my $transcript;
    if ($O_PARAM->IsDefined('long_reads_librairies'))
    {
        @a_long_lib = split(",", $O_PARAM->Get('long_reads_librairies'));
    }

    my @a_short_lib;
    if ($O_PARAM->IsDefined('short_reads_librairies'))
    {
        @a_short_lib = split(",", $O_PARAM->Get('short_reads_librairies'));
    }

    my @a_quality_files;
    if ($O_PARAM->IsDefined('quality_files'))
    {
        @a_quality_files = split(",", $O_PARAM->Get('quality_files'));
    }

    foreach my $file ((@a_short_lib, @a_long_lib, @a_quality_files))
    {
        $file = abs_path($file);
        if (!-e $file)
        {
            print STDERR $file . " is not an existing or not empty file !\n";
            exit();
        }
    }

    my @a_short_type;
    if ($O_PARAM->IsDefined('short_reads_type'))
    {
        @a_short_type = split(",", $O_PARAM->Get('short_reads_type'));
        if (scalar(@a_short_type) != 1 && scalar(@a_short_type) != scalar(@a_short_lib))
        {
            print STDERR "You have to define correctly the type of the librairies\n";
            exit();
        }
        foreach my $type (@a_short_type)
        {
            &AllowedValue($type, \@A_VELVET_TYPE, 'short_reads_type');
        }
    }

    my @a_short_format;
    if ($O_PARAM->IsDefined('short_reads_format'))
    {
        @a_short_format = split(",", $O_PARAM->Get('short_reads_format'));
        if (scalar(@a_short_format) != 1 && scalar(@a_short_format) != scalar(@a_short_lib))
        {
            print STDERR "You have to define correctly the format of the librairies\n";
            exit();
        }
        foreach my $format (@a_short_format)
        {
            &AllowedValue($format, \@A_VELVET_FORMAT, 'short_reads_format');
        }
    }

    my %h_inputs = (
                    'genome'          => $genome,
                    'genome_name'     => $genome_name,
                    'paired_ends'     => $paired_ends,
                    'annotation_file' => $annotation_file,
                    'full_length'     => $full_length,
                    'quality'         => \@a_quality_files,
                    'a_long_lib'      => \@a_long_lib,
                    'a_short_lib'     => \@a_short_lib,
                    'a_short_type'    => \@a_short_type,
                    'a_short_format'  => \@a_short_format
                    );

    return \%h_inputs;
}

=head2 procedure Velvet_Oases

 Title        : &Velvet_Oases
 Usage        :  &Velvet_Oases($repository,$ra_short_lib,$ra_short_type,$ra_short_format,$o_logger_log,$o_logger_error)
 Prerequisite : 
 Function     : Assemble reads into contigs using Velvet, and the determine transcripts using Oases.
 Args         : $repository : the work repository, $ra_short_lib : a ref to an array which contains the short librairies filepath,$ra_short_type :a ref to an array which contains the short librairies type 
 ,$ra_short_format : a ref to an array which contains the short librairies format,$o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

sub Velvet_Oases
{
    my ($repository, $ra_short_lib, $ra_short_type, $ra_short_format, $cpus, $velvet_oases_results_use, $o_logger_log,
        $o_logger_error)
      = @_;

    my $hash_length     = $O_PARAM->Get('hash_length');
    my $velvet_hash_bin = $O_PARAM->Get('Velvet_hash_script');

    my @a_short_lib    = @$ra_short_lib;
    my @a_short_type   = @$ra_short_type;
    my @a_short_format = @$ra_short_format;

    my $velvet_run_bin = $O_PARAM->Get('Velvet_run_script');

    my @a_velvet_params = $O_PARAM->GetKeys('^velvet-');
    my $velvet_params   = "";
    foreach my $param (@a_velvet_params)
    {
        my $value = $O_PARAM->Get($param);
        if ($value ne "")
        {
            $param =~ s/velvet-//;
            $velvet_params .= "-$param $value ";
        }
    }
    $velvet_params .= $O_PARAM->Get('line_command_velvet');

    my @a_oases_params = $O_PARAM->GetKeys('^oases-');
    my $oases_params   = "";
    foreach my $param (@a_oases_params)
    {
        my $value = $O_PARAM->Get($param);
        if ($value ne "")
        {
            $param =~ s/oases-//;
            $oases_params .= "-$param $value ";
        }
    }
    $oases_params .= $O_PARAM->Get('line_command_oases');
    my $oases_bin = $O_PARAM->Get('Oases_run_script');

    my $files_params         = "";
    my @a_repository         = ();
    my $paraloop_file        = "$repository/0_VELVET_OASES/paraloopfile" . time . $$;
    my $paraloop_file_handle = "";
    if ($O_PARAM->IsDefined('assemble_short_reads_lib_one_by_one'))
    {
        $paraloop_file_handle = new IO::File(">$paraloop_file")
          || Carp::croak("ERROR : Can't open the file $paraloop_file");
    }

    for (my $i = 0; $i < scalar(@a_short_lib); $i++)
    {
        my $file   = $a_short_lib[$i];
        my $type   = $a_short_type[0];
        my $format = $a_short_format[0];
        $type   = $a_short_type[$i]   if (defined $a_short_type[$i]);
        $format = $a_short_format[$i] if (defined $a_short_format[$i]);

        $file = abs_path($file);

        $files_params .= "-$type -$format $file ";

        #a paralelliser avec paraloop ?
        if ($O_PARAM->IsDefined('assemble_short_reads_lib_one_by_one'))
        {
            my ($file_name) = $file =~ /.+\/(.+)/;
            my $rep = "$repository/0_VELVET_OASES/$i.$file_name";
            push(@a_repository, "$rep/$velvet_oases_results_use");
            mkdir $rep;

            my $velvet_hash_cmd =
                "($velvet_hash_bin $rep $hash_length -$type -$format $file > Velveth.out ) 2>> $repository/"
              . $O_PARAM->Get('error_pipeline')
              . $O_PARAM->Get('error_ext');

            my $velvet_run_cmd =
                "($velvet_run_bin $rep $velvet_params > Velvetg.out ) 2>> $repository/"
              . $O_PARAM->Get('error_pipeline')
              . $O_PARAM->Get('error_ext');

            my $oases_run_cmd =
                "($oases_bin $rep $oases_params > Oases.out ) 2>> $repository/"
              . $O_PARAM->Get('error_pipeline')
              . $O_PARAM->Get('error_ext');

            print $paraloop_file_handle $velvet_hash_cmd . ";$velvet_run_cmd; $oases_run_cmd\n";

        }
    }
    $paraloop_file_handle->close if ($O_PARAM->IsDefined('assemble_short_reads_lib_one_by_one'));

    if (!$O_PARAM->IsDefined('assemble_short_reads_lib_one_by_one'))
    {
        my $velvet_hash_cmd =
            "($velvet_hash_bin $repository/0_VELVET_OASES $hash_length $files_params > Velveth.out ) 2>> $repository/"
          . $O_PARAM->Get('error_pipeline')
          . $O_PARAM->Get('error_ext');
        $o_logger_log->Trace("Launch Velvet hash : $velvet_hash_cmd\n", "*", "CMD");
        system($velvet_hash_cmd);

        my $velvet_run_cmd =
            "($velvet_run_bin $repository/0_VELVET_OASES $velvet_params > Velvetg.out ) 2>> $repository/"
          . $O_PARAM->Get('error_pipeline')
          . $O_PARAM->Get('error_ext');
        $o_logger_log->Trace("Launch Velvet run : $velvet_run_cmd\n", "*", "CMD");
        system($velvet_run_cmd);

        my $oases_run_cmd =
            "($oases_bin $repository/0_VELVET_OASES $oases_params >Oases.out ) 2>> $repository/"
          . $O_PARAM->Get('error_pipeline')
          . $O_PARAM->Get('error_ext');
        $o_logger_log->Trace("Launch Oases run : $oases_run_cmd\n", "*", "CMD");
        system($oases_run_cmd);
    }
    else
    {
        my $paraloop_cmd =
            $O_PARAM->Get('paraloop') . ' '
          . $O_PARAM->Get('paraloop_fixed_parameters')
          . ' --ncpus '
          . $cpus
          . " --input $paraloop_file > Oases.out ) 2>> $repository/"
          . $O_PARAM->Get('error_pipeline')
          . $O_PARAM->Get('error_ext');
        $o_logger_log->Trace("Launch Velvet Oases run via paraloop: $paraloop_cmd\n", "*", "CMD");
        system($paraloop_cmd);

        my $cmde_cat = "$cat " . join(" ", @a_repository) . " > $repository/0_VELVET_OASES/$velvet_oases_results_use";
        system($cmde_cat);
    }

}

=head2 function  AfterOases

 Title        : &AfterOases
 Usage        : &AfterOases($repository,$ra_long_reads,$o_logger_log,$o_logger_error)
 Prerequisite : The step Velvet_Oases has to be finished
 Function     : Convert the pasa output in file for PASA or TGICL.
 Returns      : filepath to the transcripts file result.
 Args         : $repository : the work repository, $ra_long_reads :  a ref to an array which contains the long librairies filepath, $o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

sub AfterOases
{
    my ($repository, $ra_long_reads, $velvet_oases_results_use, $o_logger_log, $o_logger_error) = @_;

    if ($O_PARAM->IsDefined('long_reads_librairies'))
    {

        #on supprime les '/' des identifiants du resultats Velvet/Oases car foutent la merde avec PASA
        my $sed_cmd = "sed -i 's/\\//_/' $repository/0_VELVET_OASES/$velvet_oases_results_use";
        $o_logger_log->Trace("Edit Velvet/Oases results : $sed_cmd\n", "*", "CMD");
        system($sed_cmd);

        #on doit merger les librairies long read avec les contigs obtenus avec velvet.
        my $cat_cmde = "$cat "
          . join(" ", @$ra_long_reads)
          . " $repository/0_VELVET_OASES/$velvet_oases_results_use > $repository/0_VELVET_OASES/AFTER_OASES.$velvet_oases_results_use+long_reads.fasta";
        $o_logger_log->Trace("Merge Oases results and long reads librairies available : $cat_cmde\n", "*", "CMD");
        system($cat_cmde);
        return "$repository/0_VELVET_OASES/AFTER_OASES.$velvet_oases_results_use+long_reads.fasta";
    }
    else
    {

        #l'entreee de Pasa ou TGICL++ sera les transcrits.
        return "$repository/0_VELVET_OASES/$velvet_oases_results_use";
    }
}

=head2 procedure Pasa

 Title        : Pasa
 Usage        : &Pasa($repository,$transcripts,$transcripts_name,$genome,$genome_name,$full_length,$o_logger_log,$o_logger_error)
 Prerequisite : none
 Function     : Launch pasa pipeline. Alignment of transcripts on the genome. The parameters of each step is modifiable via the configuration file
 Returns      : none
 Args         : $repository : the work repository,$transcripts : the path of the transcripts_file, $transcripts_name : ne name of the transcripts file, $genome : the path to the genome file, $genome_name : the name of the genome file ,$full_length : the path to the full length file ,$o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

sub Pasa
{
    my ($repository, $transcripts, $transcripts_name, $genome, $genome_name, $full_length, $o_logger_log,
        $o_logger_error)
      = @_;

    $o_logger_error->Trace("\n\n##########\n#  PASA  #\n##########\n\n", "", "ERROR");
    mkdir("$repository/1_PASA");
    chdir("$repository/1_PASA");
    system("$ln -s $transcripts .");

    #the transcripts sequences have to be clean
    my $seq_clean_bin = $O_PARAM->Get("Pasa_seqclean");
    my $cmde_seqclean =
      "$seq_clean_bin $repository/1_PASA/$transcripts_name -o $repository/1_PASA/$transcripts_name.clean 2>> $repository/"
      . $O_PARAM->Get('error_pipeline')
      . $O_PARAM->Get('error_ext');

    $o_logger_log->Trace("\n\n##########\n#  PASA  #\n##########\n", "",  "CMD");
    $o_logger_log->Trace("Launch SeqClean : $cmde_seqclean\n",       "*", "CMD");

    #Launch SeqClean
    system($cmde_seqclean);

    #build the configuration file for pasa
    my $pasa_conf_file = $O_PARAM->Get('Pasa_cfg');
    my $pasa_conf      = new IO::File(">$repository/1_PASA/alignAssembly.config")
      || Carp::croak("ERROR : Can't open the file $repository/1_PASA/pasa.alignAssembly.txt");
    my $pasa_template = new IO::File("$pasa_conf_file") || Carp::croak("ERROR : Can't open the file $pasa_conf_file");

    while (my $l = <$pasa_template>)
    {
        chomp $l;
        if ($l =~ /\<__MYSQLDB__\>/)
        {
            if ($O_PARAM->IsDefined('MYSQLDB'))
            {
                my $db = $O_PARAM->Get('MYSQLDB');
                $l =~ s/\<__MYSQLDB__\>/$db/;
            }
            else
            {
                $o_logger_error->Trace("THE NAME OF THE DATABASE HAS TO BE DEFINED IN THE CONFIGURATION FILE ",
                                       , "ERROR");
                &Usage();
            }
        }
        elsif ($l =~ /<__MAX_INTRON_LENGTH__>/)
        {
            my $max = $O_PARAM->Get('MAX_INTRON_LENGTH');
            $l =~ s/<__MAX_INTRON_LENGTH__>/$max/ if ($max =~ /^[0-9]+\s*$/);
        }
        elsif ($l =~ /<__MIN_INTRON_LENGTH__>/)
        {
            my $min = $O_PARAM->Get('MIN_INTRON_LENGTH');
            $l =~ s/<__MIN_INTRON_LENGTH__>/$min/ if ($min =~ /^[0-9]+\s*$/);
        }
        elsif ($l =~ /<__MIN_PERCENT_ALIGNED__>/)
        {
            my $min = $O_PARAM->Get('MIN_PERCENT_ALIGNED');
            $l =~ s/<__MIN_PERCENT_ALIGNED__>/$min/ if ($min =~ /^[0-9]+\s*$/);
        }
        elsif ($l =~ /<__MIN_AVG_PER_ID__>/)
        {
            my $min = $O_PARAM->Get('MIN_AVG_PER_ID');
            $l =~ s/<__MIN_AVG_PER_ID__>/$min/ if ($min =~ /^[0-9]+\s*$/);
        }

        print $pasa_conf $l . "\n";
    }
    $pasa_template->close();
    $pasa_conf->close();

    #link in the work repository
    system("$ln -s $genome .");

    my $cmde_pasa = "("
      . $O_PARAM->Get('Pasa_runpipeline')
      . " -c alignAssembly.config -C -R -g $repository/1_PASA/$genome_name -t $repository/1_PASA/$transcripts_name.clean -T -u $repository/1_PASA/$transcripts_name --USE_GMAP --APPLY_SIM4_CHASER";
    $cmde_pasa .= " -f $full_length" if (defined $full_length && $full_length ne "");
    $cmde_pasa .= " >> PASARun.out )2>> PASARun.err";

    $o_logger_log->Trace("Launch the pipeline Pasa : $cmde_pasa\n", "*", "CMD");

    # launch pasa pipeline
    system($cmde_pasa);

    #Generate a report file with the identifiers PASA
    my @a_infiles = `ls $repository/1_PASA/*_assemblies_described.txt`;
    chomp @a_infiles;
    my $infile = $a_infiles[0];

    my $file = new IO::File("$infile") || Carp::croak("-- ERROR -- Can't open the file $infile");
    my $outfile = new IO::File(">$repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_assemblies.report")
      || Carp::croak(
                   "-- ERROR -- Can't create the file $repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_assemblies.report");

    while (my $line = <$file>)
    {

        if ($line =~ /alignment_assembly:(\S+)\tcdna_accs:(\S+)/)
        {
            my $id        = $1;
            my $ests_list = $2;
            $ests_list =~ s/,/ /g;
            print $outfile ">$id\n$ests_list\n";
        }
    }
    my @a_pasa_fasta    = `ls  $repository/1_PASA/*.assemblies.fasta`;
    my $pasa_fasta_file = abs_path($a_pasa_fasta[0]);
    chomp $pasa_fasta_file;
    system("mv  $pasa_fasta_file $DATE" . "_COMPENDIUM_PASA_assemblies.fasta");

}

=head2 procedure PasaAnnotation

 Title        : PasaAnnotation
 Usage        : &PasaAnnotation($repository,$genome,$transcripts_name,$annotation_file,$o_logger_log,$o_logger_error)
 Prerequisite : none
 Function     : Launch pasa Update ans compare annotation. The parameters od the comparison can by set in the configuration file
 Returns      : none
 Args         : $repository : the work repository, $genome : the path to the genome file, $annotation_file : the path to the annotation file ,$o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

sub PasaAnnotation
{

    my ($repository, $genome_name, $transcripts_name, $annotation_file, $o_logger_log, $o_logger_error) = @_;

    chdir("$repository/1_PASA");

    #build the configuration file for annotation comparaison
    my $pasa_conf_file = $O_PARAM->Get('Pasa_annotation_cfg');
    my $pasa_conf      = new IO::File(">$repository/1_PASA/annotCompare.config")
      || Carp::croak("ERROR : Can't open the file $repository/1_PASA/annotCompare.config");
    my $pasa_template = new IO::File("$pasa_conf_file") || Carp::croak("ERROR : Can't open the file $pasa_conf_file");
    while (my $l = <$pasa_template>)
    {
        chomp $l;
        if ($l =~ /\<__MYSQLDB__\>/)
        {
            if ($O_PARAM->IsDefined('MYSQLDB'))
            {
                my $db = $O_PARAM->Get('MYSQLDB');
                $l =~ s/\<__MYSQLDB__\>/$db/;
            }
            else
            {
                $o_logger_error->Trace("THE NAME OF THE DATABASE HAS TO BE DEFINED IN THE CONFIGURATION FILE ",
                                       , "ERROR");
                &Usage();
            }
        }
        elsif ($l =~ /<__(.+)__>/)
        {
            my $param = $O_PARAM->Get($1);
            $l =~ s/<__(.+)__>/$param/ if ($param ne "");
        }

        print $pasa_conf $l . "\n";
    }
    $pasa_template->close();
    $pasa_conf->close();

#on fait tourner le script pasa_annotation_cleaner qui va enlever les mRNAs qui pose pb a PASA (plusieurs mRNAs predit a partir d'un meme gene mais sur des brins different)
    my ($annotation_name) = $annotation_file =~ /.+\/(.+)/;
    my $cmde_pasa_cleaner = "( "
      . $O_PARAM->Get('Pasa_annotation_cleaner')
      . " --gff3 $annotation_file --outfile $repository/1_PASA/$annotation_name.clean )2>> $repository/"
      . $O_PARAM->Get('error_pipeline')
      . $O_PARAM->Get('error_ext');
    $o_logger_log->Trace("Launch Pasa Cleaner Annotation : $cmde_pasa_cleaner\n", "*", "CMD");
    system($cmde_pasa_cleaner);

    my $pasa_log = "PASA.Annotation" . time . ".out";
    my $pasa_err = "PASA.Annotation" . time . ".err";

    #on charge l'annotation
    my $cmde_load = "("
      . $O_PARAM->Get('Pasa_load_annotation')
      . " -c alignAssembly.config -g $repository/1_PASA/$genome_name -P $repository/1_PASA/$annotation_name.clean";
    $cmde_load .= " >> $pasa_log )2>> $pasa_err";
    $o_logger_log->Trace("Launch Pasa Load current Annotation : $cmde_load\n", "*", "CMD");
    system($cmde_load);

    #on compare l'annotation a celle de PASA
    my $cmde_comp = "("
      . $O_PARAM->Get('Pasa_runpipeline')
      . " -c annotCompare.config -A -g $repository/1_PASA/$genome_name -t $repository/1_PASA/$transcripts_name.clean";
    $cmde_comp .= " >> $pasa_log )2>> $pasa_err";
    $o_logger_log->Trace("Launch Pasa compare annotation : $cmde_comp\n", "*", "CMD");
    system($cmde_comp);

}

=head2 procedure AnalysePasaAnnotationOutput

 Title        : AnalysePasaAnnotationOutput
 Usage        : &AnalysePasaAnnotationOutput($repository,$genome,$transcripts_name,$annotation_file,$o_logger_log,$o_logger_error)
 Prerequisite : none
 Function     : Analyse the gff3 file genrate by PASA after the comparison. 3 files are created : for each gene updated by PASA wa have, the ESTs involved, the proteine sequences, and the genomic sequence.
 Returns      : none
 Args         : $repository : the work repository, $genome : the path to the genome file, $o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

sub AnalysePasaAnnotationOutput
{
    my ($repository, $genome, $annotation_file, $o_logger_log, $o_logger_error) = @_;

    my @a_pasa_report    = `ls  $repository/1_PASA/*_COMPENDIUM_PASA_assemblies.report | sort`;
    my $pasa_report_file = abs_path($a_pasa_report[0]);

    # on va regarder le fichier genere par l'annotation et essayer de faire un fichier report avec le contenu en est
    #verifier qu'on est bien dans le bon repertoire !
    my $comparefile_gff = `ls -rta $repository/1_PASA/output.gene_structures_post_PASA_updates.*.gff3 | tail -1`;
    chomp $comparefile_gff;
    $comparefile_gff = abs_path($comparefile_gff);

    my $annotation_post_update_path =
      "$repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_gene_structures_post_PASA_updates." . time . $$ . ".gff3";

    # a rechanger en mv !
    system("cp $comparefile_gff $annotation_post_update_path");
    $o_logger_log->Trace("Rename file: mv $comparefile_gff $annotation_post_update_path\n", "*", "CMD");

    #on charge le report file : (on cree une berkeley DB ?)
    my $dbname = "/tmp/PASA_report_file" . $$ . time;
    unlink $dbname;
    tie my %h_db, "BerkeleyDB::Hash",
      -Filename => $dbname,
      -Flags    => DB_CREATE
      or die "Cannot open file $dbname: $! $BerkeleyDB::Error\n";

    my $id = "";
    my $infile = new IO::File($pasa_report_file) || Carp::croak("Cannot open the file :$pasa_report_file");
    while (my $line = <$infile>)
    {
        if ($line =~ />(\S+)\s/)
        {
            $id = $1;
        }
        else
        {
            chomp $line;
            $h_db{$id} .= $line;
        }
    }
    $infile->close;

#on parcourt le fichier de l'annotation original pour recuperer ce qui neva pas etre mis a jour par pasa... a faire ou a laisser a jerome...

    #on cree le fichier report_post_update_PASA :
    my $report_post_update =
      new IO::File(">$repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_assemblies_post_update." . time . $$ . ".report")
      || Carp::croak(  "ERROR : Can't open the file $repository/1_PASA/$DATE"
                     . "_COMPENDIUM_PASA_assemblies_post_update."
                     . time
                     . $$
                     . ".report");

#parcourt le fichier des annotations updatees et pour chacune on report le nom de gene et l'ensemble des ESTs associees aux assemblages qui ont permis de mettre a jour
    my $annotation_post_update = new IO::File($annotation_post_update_path)
      || Carp::croak("ERROR : Can't open the file $annotation_post_update_path");

    #fichier fasta nuc post update
    my $path_fasta_nuc_post_update =
      "$repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_assemblies_post_update.MRNA." . time . $$ . ".fasta";
    my $fasta_nuc = new IO::File(">$path_fasta_nuc_post_update")
      || Carp::croak("ERROR : Can't open the file $path_fasta_nuc_post_update");

    #fichier fasta prot post update
    my $fasta_prot =
      new IO::File(">$repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_assemblies_post_update.PROT." . time . $$ . ".fasta")
      || Carp::croak(
                  ">$repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_assemblies_post_update.PROT." . time . $$ . ".fasta");

    #on charge le genome dnas un hash pour pouvoir recupere la sequence genomique du gene
    my $genome_handler = new IO::File($genome) || Carp::croak("ERROR : Can't open the file genome : $genome");
    my %h_genome;

    my $seq;
    while (my $line = <$genome_handler>)
    {
        chomp $line;
        if ($line =~ />(\S+)/)
        {
            $id = $1;
        }
        else
        {
            $h_genome{$id} .= $line;
        }

        #if ($line =~ />(\S+)/)
        #{
        #	$h_genome{$id} = $seq;
        #	$seq = "";
        #	$id = $1;
        #}
        #else
        #{
        #	chomp $line;
        #	$seq .= $line;
        #}

    }
    $genome_handler->close;

    #on charge le gff3 en memoire aussi !
    my %h_dejavu;
    my %h_update_gene;
    my %h_mRNA;
    my %h_report_post_update;
    my %h_prot;
    my %h_exons;
    my $status_keeper = $O_PARAM->Get('status_pass_annotation_to_keep');
    my @a_status_keeper = split(',', $status_keeper);
    my %h_status_keeper;
    my %h_5utr;
    my %h_3utr;
    my %h_original_mrna;
    my %h_failed_status;

    foreach my $status (@a_status_keeper)
    {
        $h_status_keeper{$status} = "";
    }
    while (my $line = <$annotation_post_update>)
    {
        chomp $line;

        if ($line =~ /PASA_UPDATE: (\S+), .+status:((\[pasa:asmbl_\d+,status:\d+\],)+)/)
        {
            my $ids  = $1;
            my $asms = $2;

            $ids =~ s/_/#/g;

            $asms =~ s/\],\[/\];\[/;
            my @a_asmbl = split(";", $asms);

            my $pass_status  = 1;
            my @a_est_dejavu = ();
            foreach my $asmbl (@a_asmbl)
            {
                my ($asmbl_id, $status) = $asmbl =~ /pasa:(asmbl_\d+),status:(\d+)/;
                if (!defined $h_status_keeper{$status})
                {
                    $pass_status = 0;
                    last;
                }

                if (defined $h_dejavu{$asmbl_id})
                {
                    $o_logger_error->Trace("The PASA assembly $asmbl_id is involved in several gene update !!",
                                           "*", "ERROR");
                }
                else
                {
                    push(@a_est_dejavu, $asmbl_id);

                    #$h_dejavu{$asmbl_id}="";
                }

                #$est .= $h_db{$asmbl_id}." " if (defined $h_db{$asmbl_id});
            }
            if ($pass_status == 1)
            {
                my $est = "";
                foreach my $asmbl_id (@a_est_dejavu)
                {
                    $h_dejavu{$asmbl_id} = "";
                    $est .= $h_db{$asmbl_id} . " " if (defined $h_db{$asmbl_id});
                }
                chop $est;
                $h_report_post_update{$ids} = $est;
                $h_update_gene{$ids}        = "";
            }
            else
            {

                #my @a_ids=split('#',$ids);
                foreach my $id (split('#', $ids))
                {
                    $h_failed_status{$id} = "";
                }
            }

        }
        elsif ($line =~ /ORIGINAL: (\S+) original gene structure, not modified by PASA/)
        {
            my $id = $1;
            $h_original_mrna{$id} = "";
        }
        elsif ($line =~ /exon.+Parent=([^;\s]+)/)
        {
            my $ids = $1;
            my @a_ids = split(",", $ids);
            foreach my $id (@a_ids)
            {
                my @a_exons;
                if (defined $h_exons{$id})
                {
                    @a_exons = @{$h_exons{$id}};
                }
                push(@a_exons, $line);
                $h_exons{$id} = \@a_exons;
            }

        }
        elsif ($line =~ /#PROT (\S+)\s+\S+\s+(\S+)/)
        {
            $id = $1;
            $id =~ s/_/#/g;
            $h_prot{$id} = $2;
        }
    }

    #on parse l'annotation original pour recupere la seq des update qui ne passe passe notrer filtre sur les status
    my $annotation_original = new IO::File($annotation_file) || Carp::croak("Cannot open the file :$annotation_file");
    while (my $line = <$annotation_original>)
    {
        if ($line =~ /exon.+Parent=([^;\s]+)/)
        {
            my $ids = $1;
            my @a_ids = split(",", $ids);
            foreach my $id (@a_ids)
            {
                if (defined $h_failed_status{$id})
                {
                    my @a_exons;
                    if (defined $h_exons{$id})
                    {
                        @a_exons = @{$h_exons{$id}};
                    }
                    push(@a_exons, $line);
                    $h_exons{$id} = \@a_exons;
                }
            }
        }
    }
    $annotation_original->close;

    #LES TRANSCRITS DE L'ANNOTATION CONFORTES PAR PASA OU UPDATES
    foreach my $id_mrna (sort(keys(%h_update_gene)))
    {
        if (defined $h_exons{$id_mrna})
        {
            my @a_exons = @{$h_exons{$id_mrna}};
            my $sequence;
            foreach (my $i = 0; $i < scalar(@a_exons); $i++)
            {
                my $line = $a_exons[$i];
                my ($chro, $pos1, $pos2, $strand) = $line =~ /(\S+).+exon\s+(\d+)\s+(\d+).+([\+\-])/;
                my $exon = substr($h_genome{$chro}, $pos1, $pos2 - $pos1 + 1);

                if ($strand eq "-")
                {
                    $exon = &ReverseComplement($exon);
                }
                $sequence .= $exon;
            }
            print $fasta_nuc ">" . $id_mrna . "\n" . $sequence . "\n";
            print $fasta_prot ">$id_mrna\n" . $h_prot{$id_mrna} . "\n";
            print $report_post_update ">$id_mrna\n" . $h_update_gene{$id_mrna} . "\n";
        }
        else
        {
            $o_logger_error->Trace("Something goes wrong, no exons for updated $id_mrna\n", "*", "ERROR");
        }

    }

    $report_post_update->close;
    $annotation_post_update->close;
    $fasta_prot->close;
    $fasta_nuc->close;

    #LES ORIGINAUX DE L'ANNOTATION NON MODIFIES PAR PASA ET CEUX UPDATE MAIS DONT LE STATUS NOUS ALLAIT PAS
    my $fasta_nuc_original_path =
      "$repository/1_PASA/$DATE" . "_COMPENDIUM_PASA_assemblies_post_update.original.MRNA." . time . $$ . ".fasta";
    my $fasta_nuc_original = new IO::File(">$fasta_nuc_original_path")
      || Carp::croak("ERROR : Can't open the file $fasta_nuc_original_path");
    my @a_original_mrna_keys = keys(%h_original_mrna);
    my @a_failed_status_keys = keys(%h_failed_status);
    my @a_originaux          = ();
    push(@a_originaux, @a_original_mrna_keys, @a_failed_status_keys);
    my %h_vu;

    foreach my $id_mrna (sort @a_originaux)
    {
        if (defined $h_exons{$id_mrna} && !defined $h_vu{$id_mrna})
        {
            my @a_exons = @{$h_exons{$id_mrna}};
            my $sequence;
            foreach (my $i = 0; $i < scalar(@a_exons); $i++)
            {
                my $line = $a_exons[$i];
                my ($chro, $pos1, $pos2, $strand) = $line =~ /(\S+).+exon\s+(\d+)\s+(\d+).+([\+\-])/;
                my $exon = substr($h_genome{$chro}, $pos1, $pos2 - $pos1 + 1);
                if ($strand eq "-")
                {
                    $exon = &ReverseComplement($exon);
                }
                $sequence .= $exon;
            }
            print $fasta_nuc_original ">" . $id_mrna . "\n" . $sequence . "\n";
            $h_vu{$id_mrna} = "";
        }
        else
        {
            $o_logger_error->Trace("Something goes wrong, no exons for original $id_mrna\n", "*", "ERROR");
        }
    }
    $fasta_nuc_original->close;

    #LES NOUVEAUX ASSEMBLAGES PAR RAPPORT A L'ANNOTATION :
    #on charge le fasta des assemblages
    $dbname = "/tmp/PASA_fasta_file" . $$ . time;
    unlink $dbname;
    tie my %h_db_fasta, "BerkeleyDB::Hash",
      -Filename => $dbname,
      -Flags    => DB_CREATE
      or die "Cannot open file $dbname: $! $BerkeleyDB::Error\n";

    $id = "";
    my @a_pasa_fasta    = `ls  $repository/1_PASA/*_COMPENDIUM_PASA_assemblies.fasta`;
    my $pasa_fasta_file = abs_path($a_pasa_fasta[0]);
    chomp $pasa_fasta_file;

    my $infile_fasta = new IO::File($pasa_fasta_file) || Carp::croak("Cannot open the file :$pasa_fasta_file");
    while (my $line = <$infile_fasta>)
    {
        if ($line =~ />(\S+)/)
        {
            $id = $1;
        }
        else
        {
            $h_db_fasta{$id} .= $line;
        }
    }
    $infile_fasta->close;

    my $base_name = "$repository/1_PASA/$DATE" . "_Pasa_assemblies_not_integrated_in_annotation." . time . $$;

    #on recopie dans un fichier report les assemblages Pasa qui n'ont pas donne d'info sur l'annotation
    my $report_post_update_failed = new IO::File(">$base_name.report")
      || Carp::croak("ERROR : Can't open the file $base_name.report");

    #on recopie dans un fichier fasta les assemblages Pasa qui n'ont pas donne d'info sur l'annotation
    my $fasta_post_update_failed = new IO::File(">$base_name.fasta")
      || Carp::croak("ERROR : Can't open the file $base_name.fasta");

    foreach my $k (keys %h_db)
    {
        if (!defined $h_dejavu{$k})
        {
            print $report_post_update_failed ">" . $k . "\n" . $h_db{$k} . "\n";
            print $fasta_post_update_failed ">" . $k . "\n" . $h_db_fasta{$k};
        }
    }
    $report_post_update_failed->close;
    $fasta_post_update_failed->close;

    #POUR FINIR ON CONCATENE TOUS LES FICHIERS TRNASCRITS POUR EN FAIRE UN GENERAL
    system(  "cat $path_fasta_nuc_post_update $fasta_nuc_original_path $base_name.fasta > $repository/1_PASA/$DATE"
           . "_COMPENDIUM_PASA_post_update_ALL.MRNA.fasta")

}

sub ReverseComplement
{
    my ($seq) = @_;

    $seq = reverse($seq);
    $seq =~ tr/ACGTacgt/TGCAtcga/;

    return $seq;

}

=head2 procedure Pasa2Tgicl

 Title        : Pasa2Tgicl
 Usage        : &Pasa2Tgicl($repository,$transcripts_name,$paired_ends,$full_length,$o_logger_log,$o_logger_error)
 Prerequisite : The step PASA has to be finished
 Function     : Convert the pasa output in file for TGICL. extract the transcripts sequences that didn't match on the genome at the pasa step, and build the paired end file associated.
 Returns      : none
 Args         : $repository : the work repository, $transcripts_name : ne name of the transcripts file,$paired_ends : the path to the paired ends file ,$full_length : the path to the full length file ,$o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

# sub Pasa2Tgicl
sub AfterPasa
{
    my ($repository, $transcripts_name, $paired_ends, $full_length, $quality, $o_logger_log, $o_logger_error) = @_;

    $o_logger_error->Trace("\n\n################\n#  AFTER PASA  #\n################\n\n", "", "ERROR");
    $o_logger_log->Trace("\n\n################\n#  AFTER PASA #\n################\n\n", "", "CMD");

    #check the pasa result
    if (!-e "$repository/1_PASA/validated_transcripts.gff3")
    {
        $o_logger_error->Trace(
            "THE FILE $repository/1_PASA/validated_transcripts.gff3 doesn't not exist. Please check if the PASA step finished without error !\n",
            "*", "ERROR"
            );
        exit;
    }
    elsif (-z "$repository/1_PASA/validated_transcripts.gff3")
    {
        $o_logger_error->Trace(
            "THE FILE $repository/1_PASA/validated_transcripts.gff3 is empty. Please check if the PASA step finished without error !\n",
            "*", "ERROR"
            );
        exit;
    }

    #extract the ids that match the genome
    my $ids_ok =
      `cut -f 9 $repository/1_PASA/validated_transcripts.gff3 | perl -ne '{/Target=(\\S+)/; print \$1.\"\\n\"}' | sort -u`;
    my @a_ids_ok = split('\n', $ids_ok);
    my %h_ids_ok;
    foreach my $id (@a_ids_ok)
    {
        $h_ids_ok{$id} = '';
    }

    my %h_fulllengths;
    if (defined $full_length && $full_length ne "")
    {
        my $fl = new IO::File("$full_length") || Carp::croak("Open Failed - file : $full_length");

        while (my $line = <$fl>)
        {
            chomp $line;
            $h_fulllengths{$line} = "";
        }
    }

#create a file with the sequence that doesn't match the genome. If a sequence is full length, it has to be prefixed by 'et|' for TGICL
    my $infile = new IO::File("$repository/1_PASA/$transcripts_name.clean")
      || Carp::croak("Open Failed - file : $repository/1_PASA/$transcripts_name.clean");
    my $outfile = new IO::File(">$repository/1_PASA/AFTER_PASA_$transcripts_name")
      || Carp::croak("Open Failed - file :>$repository/1_PASA/AFTER_PASA_$transcripts_name");

    my $flag = 0;
    my %ids_tgicl;
    while (my $line = <$infile>)
    {
        if ($line =~ />(\S+)/)
        {
            my $id = $1;
            if (!defined $h_ids_ok{$id})
            {
                if (defined $h_fulllengths{$id})
                {
                    $line =~ s/>//;
                    print $outfile ">et|" . $line;
                    $flag = 1;
                }
                else
                {
                    print $outfile $line;
                    $flag = 1;
                }
                $ids_tgicl{$id} = "";
            }
            else
            {
                $flag = 0;
            }
        }
        elsif ($flag == 1)
        {
            print $outfile $line;
        }
    }

    $outfile->close;
    $infile->close;

    #build the paired ends file for TGICL
    my $paired_end_file = "";
    if (defined $paired_ends && $paired_ends ne "")
    {
        my $paired_end_file = "$repository/1_PASA/AFTER_PASA_$transcripts_name.pairedends";
        my $file_pe_out     = new IO::File(">$paired_end_file") || Carp::croak("Open Failed - file : $paired_end_file");
        my $file_pe_handle  = new IO::File($paired_ends) || Carp::croak("Open Failed - file : $paired_ends");

        while (my $line = <$file_pe_handle>)
        {
            my @a_paired = split("\t", $line);
            my $pair1    = $a_paired[0];
            my $pair2    = $a_paired[1];
            chomp $pair2;
            chomp $pair1;

            if ((defined $ids_tgicl{$pair1}) && (defined $ids_tgicl{$pair2}))
            {
                print $file_pe_out "$pair1\t$pair2\n";
            }
        }
        $file_pe_handle->close;
        $file_pe_out->close;
    }

    #build the quality file for TGICL
    my $quality_file = "";
    if (defined $quality && $quality ne "")
    {
        $quality_file = "$repository/1_PASA/AFTER_PASA_$transcripts_name.quality";
        my $file_qu_out    = new IO::File(">$quality_file") || Carp::croak("Open Failed - file : $quality_file");
        my $file_qu_handle = new IO::File($quality)         || Carp::croak("Open Failed - file : $quality");

        my $in = 0;
        while (my $line = <$file_qu_handle>)
        {
            if ($line =~ />(\S+)/)
            {
                my $id = $1;
                if (defined $ids_tgicl{$id})
                {
                    if (defined $h_fulllengths{$id})
                    {
                        $line =~ s/>//;
                        print $file_qu_out ">et|" . $line;
                    }
                    else
                    {
                        print $file_qu_out $line;
                    }

                    $in = 1;
                }
                else
                {
                    $in = 0;
                }
            }
            elsif ($in == 1)
            {
                print $file_qu_out $line;
            }
        }
        $file_qu_handle->close;
        $file_qu_out->close;
    }

    #Quel est le fichier de mRNA ?
    my $mRNA = "";
    my @a_fasta_assemblies_update =
      `find $repository/1_PASA -name \*_COMPENDIUM_PASA_post_update_ALL.MRNA.fasta | sort`;
    if (defined $a_fasta_assemblies_update[0])
    {
        $mRNA = $a_fasta_assemblies_update[0];
    }
    else
    {
        my @a_fasta_assemblies = `find $repository/1_PASA -name \*_COMPENDIUM_PASA_assemblies.fasta`;
        if (defined defined $a_fasta_assemblies[0])
        {
            $mRNA = $a_fasta_assemblies[0];
        }
    }
    chomp $mRNA;
    return ("$repository/1_PASA/AFTER_PASA_$transcripts_name", $paired_end_file, $quality_file, $mRNA);

}

=head2 procedure Tgicl

 Title        : Tgicl
 Usage        : &Tgicl($repository,$transcripts_name,$o_logger_log,$o_logger_error)
 Prerequisite : existence of the files create by Pasa2tgicl : $repository/2_TGICL/tgicl_$transcripts_name,  $repository/2_TGICL/tgicl_$transcripts_name.pairedends
 Function     : Launch the TGICLplus pipeline. The transcripts are clustered.
 Returns      : none
 Args         : $repository : the work repository, $transcripts_name : the name of the transcripts file,$o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

sub Tgicl
{
    my ($repository, $infile, $transcripts_name, $paired_ends, $quality, $cpu, $o_logger_log, $o_logger_error) = @_;

    $o_logger_log->Trace("\n\n###########\n#  TGICL  #\n###########\n", "", "CMD");
    $o_logger_error->Trace("\n\n###########\n#  TGICL  #\n###########\n", "", "ERROR");

    mkdir("$repository/2_TGICL") if (!-d "$repository/2_TGICL");
    chdir("$repository/2_TGICL");

    system("$ln -s $infile $repository/2_TGICL/tgicl_$transcripts_name")
      if (!-e "$repository/2_TGICL/tgicl_$transcripts_name");

    #build command line
    my $cmde_tgicl .= "(" . $O_PARAM->Get('Tgicl_script') . " -i $repository/2_TGICL/tgicl_$transcripts_name -c $cpu ";

    $cmde_tgicl .= " -P $paired_ends " if (defined $paired_ends && -e $paired_ends);
    $cmde_tgicl .= " -q $quality "     if (defined $quality     && -e $quality);
    if ($O_PARAM->IsDefined('blast_min_percent_overlap'))
    {
        my $min = $O_PARAM->Get('blast_min_percent_overlap');
        $cmde_tgicl .= " -p $min" if ($min =~ /\d+/);
    }
    if ($O_PARAM->IsDefined('blast_min_length_overlap'))
    {
        my $min = $O_PARAM->Get('blast_min_length_overlap');
        $cmde_tgicl .= " -l $min" if ($min =~ /\d+/);
    }

    my $cap3_param = "";
    if ($O_PARAM->IsDefined('cap3_min_percent_overlap'))
    {
        my $min = $O_PARAM->Get('cap3_min_percent_overlap');
        $cap3_param .= " -p $min" if ($min =~ /\d+/);
    }
    if ($O_PARAM->IsDefined('cap3_min_length_overlap'))
    {
        my $min = $O_PARAM->Get('cap3_min_length_overlap');
        $cap3_param .= " -o $min" if ($min =~ /\d+/);
    }
    if ($O_PARAM->IsDefined('tgicl_cfg'))
    {
        my $file = $O_PARAM->Get('tgicl_cfg');
        if (defined $file && $file ne "")
        {
            if (-e $file)
            {
                $cmde_tgicl .= " -k $file";
            }
            else
            {
                $o_logger_error->Trace(
                                     "\n\n error tgicl configuration file -> $file is not the name of an existing file",
                                     "", "ERROR");
            }
        }
    }

    $cmde_tgicl .= " -r " . $O_PARAM->Get('tgicl_restart') if ($O_PARAM->IsDefined('tgicl_restart'));
    $cmde_tgicl .= " -O '$cap3_param'" if ($cap3_param ne "");
    $cmde_tgicl .= " -S " if ($O_PARAM->Get('tgicl_simplify_step') =~ /^true$/i);
    $cmde_tgicl .=
        " >> $repository/"
      . $O_PARAM->Get('log_pipeline')
      . $O_PARAM->Get('log_ext')
      . " )2>> $repository/"
      . $O_PARAM->Get('error_pipeline')
      . $O_PARAM->Get('error_ext');

    $o_logger_log->Trace("Launch TGICL command : $cmde_tgicl\n", "*", "CMD");

    #launch tgiclplus
    system($cmde_tgicl);

}

sub AfterTgicl
{

    my ($repository, $transcripts_name, $o_logger_log, $o_logger_error) = @_;

    my $FrameDP_level = $O_PARAM->Get('FrameDP_level');
    my $FrameDP_input = "";

#on recupere les contigs au dernier niveau pas interressant en fait ne representent pas les vrais contigs. Il faut prend ceux dans le fichier .fasta qui ont un count > 1
    if ($FrameDP_level eq "contigs")
    {
        my $infile = new IO::File("$repository/2_TGICL/$transcripts_name")
          || Carp::croak("Cannot open the file $repository/2_TGICL/$transcripts_name");
        my $outfile = new IO::File(">$repository/4_FRAMEDP/$transcripts_name.CONTIGS.fasta")
          || Carp::croak("Cannot open the file >$repository/4_FRAMEDP/$transcripts_name.CONTIGS.fasta");

        #le fichier est trie c'est pour cela qu'on peut le faire comme ca (en sortant des que l'on a un count a 1)
        while (my $line = <$infile>)
        {
            if ($line =~ /^>.+count=(\d+)/)
            {
                last if ($1 == 1);

            }
            print $outfile $line;

        }
        $FrameDP_input = "$repository/4_FRAMEDP/$transcripts_name.CONTIGS.fasta";
    }
    else
    {
        $FrameDP_input = "$repository/2_TGICL/$transcripts_name";
        system("ln -s $FrameDP_input");
        $FrameDP_input = "$repository/4_FRAMEDP/$transcripts_name";

    }

    $o_logger_log->Trace("AFTER TGICL ALL SEQUENCES : $FrameDP_input\n", "*", "CMD");
    return $FrameDP_input;
}

=head2 procedure FrameDP

 Title        : FrameDP
 Usage        : &FrameDP($repository,$o_logger_log,$o_logger_error)
 Prerequisite : existence of the files create by Tgicl : $repository/2_TGICL/asm_*/contigs
 Function     : Launch FramdeDP. Predict peptides from the Tgicl contigs
 Returns      : none
 Args         : $repository : the work repository, $o_logger_log : object logger,$o_logger_error : object logger
 Globals      : $O_PARAM

=cut 

sub FrameDP
{
    my ($repository, $infile, $o_logger_log, $o_logger_error) = @_;

    $o_logger_log->Trace("\n\n#############\n#  FrameDP  #\n#############\n", "", "CMD");
    $o_logger_error->Trace("\n\n#############\n#  FrameDP  #\n#############\n", "", "ERROR");

    mkdir("$repository/4_FRAMEDP") if (!-d "$repository/4_FRAMEDP");
    chdir("$repository/4_FRAMEDP");

    my $cfg_framedp = $O_PARAM->Get('FrameDP_cfg');
    if (!defined $cfg_framedp || !-e $cfg_framedp)
    {
        $o_logger_error->Trace("parameter FrameDP_cfg have to be defined in the compendium_program configuration file",
                               "", "ERROR");
        exit();
    }

    my $cmd_configure =
        "sed \"s/--ncpus 2/--ncpus "
      . $O_PARAM->Get('cpus') . "/g\" "
      . $O_PARAM->Get('FrameDP_cfg')
      . ">$repository/4_FRAMEDP/FrameDP.cfg";
    $o_logger_log->Trace("FrameDP configuration file : $cmd_configure\n", "*", "CMD");
    system($cmd_configure);

    my $cmde_framedp = "( "
      . $O_PARAM->Get('FrameDP_script')
      . " --cfg $repository/4_FRAMEDP/FrameDP.cfg --infile $infile --outdir $repository/4_FRAMEDP";
    $cmde_framedp .=
        " >> $repository/"
      . $O_PARAM->Get('log_pipeline')
      . $O_PARAM->Get('log_ext')
      . " )2>> $repository/"
      . $O_PARAM->Get('error_pipeline')
      . $O_PARAM->Get('error_ext');

    $o_logger_log->Trace("Launch FrameDP command : $cmde_framedp\n", "*", "CMD");
    system($cmde_framedp);

}

sub Soap
{
    my ($repository, $transcripts, $ra_short_reads_lib, $cpus, $o_logger_log, $o_logger_error) = @_;

    $o_logger_log->Trace("\n\n##########\n#  Soap  #\n##########\n", "", "CMD");
    $o_logger_error->Trace("\n\n##########\n#  Soap  #\n##########\n", "", "ERROR");

    my @a_soap_params = $O_PARAM->GetKeys('^soap-');
    my $soap_params   = "";
    foreach my $param (@a_soap_params)
    {
        my $value = $O_PARAM->Get($param);
        if ($value ne "")
        {
            $param =~ s/soap-//;
            $soap_params .= "-$param $value ";
        }
    }
    $soap_params .= $O_PARAM->Get('line_command_soap');
    my $soap_script = $O_PARAM->Get('Mapreads_script');

    my @a_short_lib  = @$ra_short_reads_lib;
    my @a_repository = ();

    #les faire une par une par défaut ?
    my $paraloop_file        = "$repository/3_SOAP/paraloopfile" . time . $$;
    my $paraloop_file_handle = new IO::File(">$paraloop_file")
      || Carp::croak("ERROR : Can't open the file $paraloop_file");
    for (my $i = 0; $i < scalar(@a_short_lib); $i++)
    {
        my $file = $a_short_lib[$i];
        $file = abs_path($file);
        my ($file_name) = $file =~ /.+\/(.+)/;
        my $rep         = "$repository/3_SOAP/$i.$file_name";
        my $output      = $rep . "/Soap_results";
        mkdir $rep;

        my $soap_cmd =
          "($soap_script --lib $file --reference $transcripts --program soap --outfile $output --keep=best ";
        $soap_cmd .= " --program_param \"$soap_params\" " if ($soap_params ne "");
        $soap_cmd .=
          "> $rep/Soap.out ) 2>> $repository/" . $O_PARAM->Get('error_pipeline') . $O_PARAM->Get('error_ext');

        my $convert_script = $O_PARAM->Get('Mapreads_convert');
        my $convert_cmd =
            "($convert_script --m8 $output.best --outfile $output.report> $rep/Soap_convert.out ) 2>> $repository/"
          . $O_PARAM->Get('error_pipeline')
          . $O_PARAM->Get('error_ext');
        push(@a_repository, $output . ".report");
        print $paraloop_file_handle "$soap_cmd;$convert_cmd\n";
    }
    $paraloop_file_handle->close;

    my $paraloop_cmd = "("
      . $O_PARAM->Get('paraloop') . ' '
      . $O_PARAM->Get('paraloop_fixed_parameters')
      . ' --ncpus '
      . $cpus
      . " --input $paraloop_file >> $repository/"
      . $O_PARAM->Get('log_pipeline')
      . $O_PARAM->Get('log_ext')
      . " ) 2>> $repository/"
      . $O_PARAM->Get('error_pipeline')
      . $O_PARAM->Get('error_ext');
    $o_logger_log->Trace("Launch Mapreads (SOAP) run/convert via paraloop: $paraloop_cmd\n", "*", "CMD");
    system($paraloop_cmd);

    #on concatene les resultats report :
    my $cat_cmd = "$cat " . join(" ", @a_repository) . ">$repository/3_SOAP/Result_mapping.report";
    $o_logger_log->Trace("Launch Cat command : $cat_cmd\n", "*", "CMD");
    system($cat_cmd);

    #on va compresser les fichiers .raw qui prennent une place enorme !
    my $gzip_cmd = "$gzip -f $repository/3_SOAP/*/*.raw $repository/3_SOAP/*/*.raw.unmapped &";
    $o_logger_log->Trace("Launch compression command : $gzip_cmd\n", "*", "CMD");
    system($gzip_cmd);
}

=head2 procedure PasaStat

 Title        : PasaStat
 Usage        : &PasaStat($repository,$transcripts_name,$paired_ends,$stat_path)
 Prerequisite : existence of the file $repository/1_PASA/validated_transcripts.gff3
 Function     : Perform some statistiques on the pasa results
 Returns      : none
 Args         : $repository : the work repository, $transcripts_name : the name of the transcripts file, $paired_ends : the path to the paired ends file,$stat_path : path to the file stat 
 Globals      : $O_PARAM

=cut

sub PasaStat
{
    my ($repository, $transcripts_name, $paired_ends, $stat_path, $o_logger_error) = @_;

    my $stat = new IO::File(">>$stat_path") || Carp::croak("-- stat -- cannot open the stat File : $stat_path");

    if (-e "$repository/1_PASA/validated_transcripts.gff3")
    {
        my $cmde =
          "cut -f 9 $repository/1_PASA/validated_transcripts.gff3 | perl -ne '{/Target=(\\S+)/; print \$1.\"\\n\"}' | sort -u";
        my @a_validated_ids = split('\n', `$cmde`);

        my $transcripts_ids = "";
        $transcripts_ids = `grep '>' $repository/1_PASA/$transcripts_name.clean`
          if (-e "$repository/1_PASA/$transcripts_name.clean");
        my @a_transcripts_ids = split('\n', $transcripts_ids);

        my $nb_clean_ids           = scalar(@a_transcripts_ids);
        my $nb_seq_non_alignes     = `grep -c 'Paths (0)' $repository/1_PASA/output.tophits`;
        my $nb_transcripts_alignes = ($nb_clean_ids - $nb_seq_non_alignes);

        my $nb_assemblies = `grep -c "// assembly" $repository/1_PASA/output.alignment_assemblies.out`;

        my $pe_s = "";
        if (defined $paired_ends && -e $paired_ends)
        {
            my $file_pe_handle = new IO::File($paired_ends) || Carp::croak("Open Failed - file : $paired_ends");

            my %h_paired_ends;
            my $compt = 0;
            while (my $line = <$file_pe_handle>)
            {
                my @a_paired = split("\t", $line);
                my $pair1    = $a_paired[0];
                my $pair2    = $a_paired[1];
                chomp $pair2;
                chomp $pair1;

                my $p1 = "";
                my $p2 = "";
                ($p1) = $transcripts_ids =~ /$pair1/;
                ($p2) = $transcripts_ids =~ /$pair2/;

                if (($p1 ne "") && ($p2 ne ""))
                {
                    $h_paired_ends{$pair1} = $pair2;
                    $h_paired_ends{$pair2} = $pair1;
                    $compt++;
                }
            }
            $file_pe_handle->close;

            my %h_ids_pe;
            my %h_unique_pd;
            my $compt2 = 0;
            my $compt3 = 0;
            foreach my $id (@a_validated_ids)
            {
                if (defined $h_paired_ends{$id})
                {
                    $compt3++;
                    my $pair = $h_paired_ends{$id};
                    if (!defined $h_unique_pd{$pair})
                    {
                        $h_unique_pd{$id} = $id;
                    }
                    else
                    {
                        delete $h_unique_pd{$id};
                        delete $h_unique_pd{$pair};
                        $compt2++;
                    }
                }
            }

            my $compt4 = scalar(keys %h_unique_pd);
            $pe_s .= "Total paired ends (after seqclean) : $compt\n";
            $pe_s .= "Total valid paired end (aligned and validated) : $compt2 ("
              . sprintf("%.2f", $compt2 / $compt * 100) . "%)\n";
            $pe_s .= "Total paired end broken : $compt4 (" . sprintf("%.2f", $compt4 / $compt * 100) . "%)\n";
        }

        my $nb_validated_ids = scalar(@a_validated_ids);
        print $stat "\n############\n#   PASA   #\n############\n\n";

        print $stat "Total transcripts seq (after seqclean): $nb_clean_ids\n";
        print $stat "Number transcripts with any alignment : "
          . $nb_transcripts_alignes . " ("
          . sprintf("%.2f", $nb_transcripts_alignes / $nb_clean_ids * 100) . " %)\n";
        print $stat "Total valid alignments : $nb_validated_ids ( "
          . sprintf("%.2f", $nb_validated_ids / $nb_clean_ids * 100) . "%)\n";
        print $stat "Total assemblies : $nb_assemblies\n";
        print $stat "# -- stat paired_ends -- #\n";

        if (defined $paired_ends && -e $paired_ends)
        {
            print $stat $pe_s;
        }
        else
        {
            print $stat "No paired ends file\n";
        }
        $stat->close();
    }
    else
    {
        $o_logger_error->Trace("###  PASA  : no file result $repository/1_PASA/validated_transcripts.gff3 found !! \n",
                               "", "ERROR");
        exit();
    }
}

=head2 procedure TgiclStat

 Title        : TgiclStat
 Usage        : &TgiclStat($repository,$transcripts_name,$stat_path)
 Prerequisite : existence of the results tgicl
 Function     : Perform some statistiques on the tgicl results
 Returns      : none
 Args         : $repository : the work repository, $transcripts_name : the name of the transcripts file, $stat_path : path to the file stat 
 Globals      : $O_PARAM

=cut

sub TgiclStat
{
    my ($repository, $transcripts, $transcripts_name, $stat_path, $o_logger_error) = @_;

    if (-e "$repository/2_TGICL/tgicl_$transcripts_name.tgicl++.consensus.fasta")
    {
        my $nb_transcrits = `grep -c ">" $transcripts`;

        my $nb_contig = 0;
        my $arg_cpu   = $O_PARAM->Get('cpus');
        my $cpus      = 1;
        if (-e $arg_cpu)
        {
            $cpus = `wc -l $arg_cpu`;
            $cpus =~ /(\d+)\s/;
        }
        else
        {
            $cpus = $arg_cpu;
        }

        my $nb_singletons = 0;
        my $cmde          = `egrep -c 'count=1 ' tgicl_$transcripts_name.tgicl++.consensus.fasta`;
        ($nb_singletons) = $cmde =~ /(\d+).*/;

        my $cmde2 = `egrep -c '>' $repository/2_TGICL/tgicl_$transcripts_name.tgicl++.consensus.fasta`;
        my ($nb_seq) = $cmde2 =~ /(\d+).*/;

        chomp $nb_transcrits;
        chomp $nb_singletons;
        chomp $nb_contig;

        my $stat = new IO::File(">>$stat_path") || Carp::croak("-- stat -- cannot open the stat File : $stat_path");

        if ($nb_transcrits != 0)
        {
            print $stat "\n#############\n#   TGICL   #\n#############\n\n";
            print $stat "Total transcripts seq : $nb_transcrits\n";
            print $stat "Number transcripts clusterised : "
              . ($nb_transcrits - $nb_singletons) . " ("
              . sprintf("%.2f", ($nb_transcrits - $nb_singletons) / $nb_transcrits * 100) . "%)\n";
            print $stat "Number singletons (never clusterised at any step) : $nb_singletons ("
              . sprintf("%.2f", $nb_singletons / $nb_transcrits * 100) . "%)\n";
            print $stat "Number contigs in final file : "
              . ($nb_seq - $nb_singletons) . " ("
              . sprintf("%.2f", ($nb_seq - $nb_singletons) / $nb_seq * 100) . "%)\n";
        }
        $stat->close();
    }
    else
    {
        $o_logger_error->Trace(
            "###  TGICL  : no file result  $repository/2_TGICL/tgicl_$transcripts_name.tgicl++.consensus.fasta found !!\n",
            "", "ERROR"
            );
        exit();
    }
}

#
#CORRECTION ET REVERSECOMPLEMENT
#

sub FrameDPBuildCorrectedClustersDB
{
    my ($repository, $o_logger_log, $o_logger_error) = @_;

    $o_logger_log->Trace("\n\n#############\n#  FrameDPBuildCorrectedClustersDB  #\n#############\n", "", "CMD");
    $o_logger_error->Trace("\n\n#############\n#  FrameDPBuildCorrectedClustersDB  #\n#############\n", "", "ERROR");

    my $cfg_framedp = $O_PARAM->Get('FrameDP_cfg');
    if (!defined $cfg_framedp || !-e $cfg_framedp)
    {
        $o_logger_error->Trace("parameter FrameDP_cfg have to be defined in the compendium_program configuration file",
                               "", "ERROR");
        exit();
    }
    my $o_local_param = New ParamParser($cfg_framedp);
    $o_local_param->SetUnlessDefined('unixcat',  'cat');
    $o_local_param->SetUnlessDefined('unixcp',   'cp');
    $o_local_param->SetUnlessDefined('unixfind', 'find');

    my $unixcat  = $o_local_param->Get('unixcat');
    my $unixcp   = $o_local_param->Get('unixcp');
    my $unixfind = $o_local_param->Get('unixfind');

    my $framed_directory = "$repository/4_FRAMEDP";
    if (!-d $framed_directory)
    {
        $o_logger_error->Trace("framed_directory  4_FRAMEDP not found in $repository", "", "ERROR");
        exit();
    }

    my @a_subdirectories = `$unixfind $framed_directory -maxdepth 1 -name 'FDP?????'`;
    chomp @a_subdirectories;
    my $paraloop_file        = "$repository/4_FRAMEDP/paraloopfile" . time . $$;
    my $paraloop_file_handle = new IO::File(">$paraloop_file")
      || Carp::croak("ERROR : Can't open the file $paraloop_file");
    my @a_corrected_files = ();
    foreach my $subdirectory (@a_subdirectories)
    {
        my @a_cluster_dirs = `$unixfind $subdirectory -mindepth 1 -type d`;
        chomp @a_cluster_dirs;
        foreach my $cluster_dir (@a_cluster_dirs)
        {
            my ($acc) = ($cluster_dir =~ /.+\/(.+)$/);
            my $framed_mode = $o_local_param->Get('framed_mode');
            if ($framed_mode !~ /fixfs/i or $framed_mode !~ /revcomp/i)
            {

                print $paraloop_file_handle "$unixcp $cluster_dir/$acc $cluster_dir/$acc.cor;";
                if (-e "$cluster_dir/$acc.gff3")
                {
                    print $paraloop_file_handle
                      "$ENV{COMPENDIUM}/bin/FrameDP/bin/gff2fasta.pl --outdir $cluster_dir --outseq $cluster_dir/$acc.cor\n";
                }
                push(@a_corrected_files, "$cluster_dir/$acc.cor");
            }
            else
            {
                push(@a_corrected_files, "$cluster_dir/$acc");
            }

        }
    }
    $paraloop_file_handle->close;
    if (-s $paraloop_file)
    {
        mkdir "$repository/4_FRAMEDP/FrameDPBuildCorrectedClustersDB";
        if (!-d "$repository/4_FRAMEDP/FrameDPBuildCorrectedClustersDB")
        {
            $o_logger_error->Trace("$repository/4_FRAMEDP/FrameDPBuildCorrectedClustersDB not created", "", "ERROR");
            exit();
        }

        chdir "$repository/4_FRAMEDP/FrameDPBuildCorrectedClustersDB";
        my $cpus = $O_PARAM->Get('cpus');
        my $paraloop_cmd =
            "cd $repository/4_FRAMEDP/FrameDPBuildCorrectedClustersDB;"
          . $O_PARAM->Get('paraloop') . ' '
          . $O_PARAM->Get('paraloop_fixed_parameters')
          . ' --ncpus '
          . $cpus
          . " --input $paraloop_file --wait > .out ) 2>> $repository/"
          . $O_PARAM->Get('error_pipeline')
          . $O_PARAM->Get('error_ext');
        $o_logger_log->Trace("Launch FrameDPBuildCorrectedClustersDB via paraloop: $paraloop_cmd\n", "*", "CMD");
        system($paraloop_cmd);
    }

    foreach my $corrected_files (sort @a_corrected_files)
    {
        system("$unixcat $corrected_files >> $framed_directory/clusters.corrected.fasta");
    }
    return;
}

sub FrameDPTarFrameDPictures
{
    my ($repository, $o_logger_log, $o_logger_error) = @_;

    $o_logger_log->Trace("\n\n#############\n#  FrameDPTarFrameDPictures  #\n#############\n", "", "CMD");
    $o_logger_error->Trace("\n\n#############\n#  FrameDPTarFrameDPictures  #\n#############\n", "", "ERROR");

    my $cfg_framedp = $O_PARAM->Get('FrameDP_cfg');
    if (!defined $cfg_framedp || !-e $cfg_framedp)
    {
        $o_logger_error->Trace("parameter FrameDP_cfg have to be defined in the compendium_program configuration file",
                               "", "ERROR");
        exit();
    }

    my $o_local_param = New ParamParser($cfg_framedp);
    $o_local_param->SetUnlessDefined('unixfind', 'find');
    $o_local_param->SetUnlessDefined('unixcp',   'cp');
    $o_local_param->SetUnlessDefined('unixrm',   'rm');


    my $unixfind = $o_local_param->Get('unixfind');
    my $unixcp   = $o_local_param->Get('unixcp');
	my $unixrm   = $o_local_param->Get('unixrm');
    
    $o_logger_log->Trace("Launch `$unixfind $repository/4_FRAMEDP/FDP?????/ -name '*.png' \n", "*", "CMD");
    my @a_files = `$unixfind $repository/4_FRAMEDP/FDP?????/ -name '*.png'`;
    chomp @a_files;

    my %h_files = ();
    foreach my $file (@a_files)
    {
        my $basename = basename($file);
        my ($acc) = ($basename =~ /^(\w+)/);
        $h_files{$acc} = 0 unless defined($h_files{$acc});
        $h_files{$acc}++;
        mkdir "$repository/4_FRAMEDP/FrameDPictures/" unless (-d "$repository/4_FRAMEDP/FrameDPictures/");
        system("$unixcp $file $repository/4_FRAMEDP/FrameDPictures/$acc.$h_files{$acc}.png");
    }

	if (-d "$repository/4_FRAMEDP/FrameDPictures/")
	{
	    $O_PARAM->SetUnlessDefined('unix_tar',  'tar');
	    $O_PARAM->SetUnlessDefined('unix_gzip', 'gzip');
	    my $unixtar  = $O_PARAM->Get('unix_tar');
	    my $unixgzip = $O_PARAM->Get('unix_gzip');
	    chdir "$repository/4_FRAMEDP/";
	    $o_logger_log->Trace(
	        "cd $repository/4_FRAMEDP/;$unixtar cf FrameDPictures.tar FrameDPictures/;$unixgzip FrameDPictures.tar;$unixrm -rf $repository/4_FRAMEDP/FrameDPictures\n",
	        "*", "CMD"
	        );
	    system(
	        "cd $repository/4_FRAMEDP/;$unixtar cf FrameDPictures.tar FrameDPictures/;$unixgzip FrameDPictures.tar;$unixrm -rf $repository/4_FRAMEDP/FrameDPictures"
	        );
	}
    return;
}

=head2 procedure FrameDPStat

 Title        : FrameDPStat
 Usage        : &FrameDPStat($repository,$transcripts_name,$stat_path)
 Prerequisite : existence of the results FrameDP
 Function     : Perform some statistiques on the framdp results
 Returns      : none
 Args         : $repository : the work repository, $stat_path : path to the file stat 
 Globals      : $O_PARAM

=cut

sub FrameDPStat
{
    my ($repository, $stat_path) = @_;

    my @a_summary_file = `find $repository/4_FRAMEDP/ -name \\*.summary`;
    chomp(@a_summary_file);
    my $summary_file = $a_summary_file[0];

    my $stat = new IO::File(">>$stat_path") || Carp::croak("-- stat -- cannot open the stat File : $stat_path");
    if (defined $summary_file && -e $summary_file)
    {
        my $infile = new IO::File("$summary_file") || Carp::croak("Open Failed - file : $summary_file");

        my %h_present;
        my %h_nb_prediction;
        my %h_brin;
        my %h_brin_different;
        while (my $line = <$infile>)
        {
            chomp $line;
            my @a_elts = split("\t", $line);
            my $id     = $a_elts[0];
            my $brin   = $a_elts[5];

            if (!defined $h_present{$id})
            {
                $h_present{$id} = $line . "\n";
                $h_brin{$id}    = $brin;
            }
            else
            {
                my $l = $h_present{$id};
                $h_present{$id} = $l . $line . "\n";
                if (defined $h_nb_prediction{$id})
                {
                    $h_nb_prediction{$id} = $h_nb_prediction{$id} + 1;
                }
                else
                {
                    $h_nb_prediction{$id} = 2;
                }
                if ($brin ne $h_brin{$id})
                {
                    $h_brin{$id}           = 0;
                    $h_brin_different{$id} = "";
                }
            }
        }
        $infile->close();

        my $nb_contigs_multi        = scalar(keys %h_nb_prediction);
        my $nb_contigs_multi_strand = scalar(keys %h_brin_different);
        my $nb_contigs              = scalar(keys %h_present);

        print $stat "\n##############\n#   FrameDP   #\n##############\n\n";
        print $stat "Total sequences with at least one peptide predicted : $nb_contigs\n";
        print $stat "Number sequences with more than one peptide predicted : $nb_contigs_multi ("
          . sprintf("%.2f", $nb_contigs_multi / $nb_contigs * 100) . "%)\n"
          if ($nb_contigs != 0);
        print $stat
          "Number sequences with more than one peptide predicted, not on the same strand : $nb_contigs_multi_strand ("
          . sprintf("%.2f", $nb_contigs_multi_strand / $nb_contigs * 100)
          . "% of total contigs)\n"
          if ($nb_contigs != 0);
    }
    else
    {
        print $stat "\n##############\n#   FrameDP   #\n##############\n\n";
        print $stat "no summary file found !\n";
    }
    $stat->close();
}

sub AllowedValue
{
    my ($value, $ra_allowed_values, $param) = @_;

    my $find = 0;
    foreach my $val (@$ra_allowed_values)
    {
        if ("$val" eq "$value")
        {
            $find = 1;
            last;
        }
    }

    if ($find == 0)
    {
        my $values = join(",", @$ra_allowed_values);
        print STDERR "$value is not correct (param : $param). Allowed values are : [$values]\n";
        exit;
    }

}

sub Usage
{
    my $allowed        = join(",", @A_STEPS);
    my $allowed_format = join(",", @A_VELVET_FORMAT);
    my $allowed_type   = join(",", @A_VELVET_TYPE);

    print STDOUT <<END;

Usage:

     $0 --genome_file <file> --short_reads_librairies<file(s)> --short_reads_type<string> --short_reads_format<string> --long_reads_librairies<file(s)> 
        --cfg_file <file>  --work_repository <dir> [--assemble_short_reads_lib_one_by_one] [--full_lengths_file <file>]  [--paired_ends_file <file>] [--step  <string>] 
        [--cpus <integer|filename>] [--tgicl_restart <integer>] [--annotation_file <filename>] [--update_annotation_pasa_only]

    Mandatory
        --genome_file                          : format fasta file of the genome
        --short_reads_librairies               : path to file(s). If several file, it should have separateb by a comma, without space
        --cfg_file                             : the configuration file, in wich parameters of Pasa and TGICL are modifiable
        --work_repository                      : the path of the work directory
        --long_reads_librairies                : path to file(s). If several file, it should have separateb by a comma, without space

    Optional
        --short_reads_type                     : allowed values : $allowed_type [mandatory if short_reads_librairies defined for velvet_oases step]
        --short_reads_format                   : allowed values : $allowed_format [mandatory if short_reads_librairies defined for velvet_oases step]
        --quality_file                         : quality file of the transcripts
        --full_length_file                     : file with the identifiers of the transcripts known as full lengths (this ids have to be present in the transcripts file !)
        --paired_ends_file                     : file with the identifiers of couple of transcripts known as paired ends (the ids of the couple have to be on the same line)
                           eg : AL367179	AL367180
                                MtRH15N17A1	MtRH15N17N1
        --annotation_file                      : format gff3 file annotation of the genome
        --update_annotation_pasa_only          : if defined, the script will only update the annotation and compare it with PASA annotation
        --step                                 : run particular step of the pipeline, allowed values : $allowed
        --cpus                                 : either a number of cpus or a file containing the list of pvm nodes
        --tgicl_restart                        : if tgicl is use with the simplification_step (see configuration file), you can define a restart step
        --assemble_short_reads_lib_one_by_one 
END
}
