#!/usr/bin/perl

# Copyright INRA/CNRS
#
# sebastien.carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

	TGICL_clustering_pairendends_add.pl
 
=head1 SYNOPSIS

 TGICL_clustering_pairendends_add.pl  --hitsort_files <file or filelist> ,outfile <file>, paired_file <file>, transcripts_file <file>

=head1 DESCRIPTION

  TGICL_clustering_pairendends_add.pl :
	Create a format pairwise comparison file in order to add the paired end information before the tgicplus clustering step.

TGICL_clustering_pairendends_add.pl --hitsort_files <file or filelist> ,outfile <file>, paired_file <file>, transcripts_file <file>
  Mandatory
	transcripts_file : fasta file with all the transcripts sequences (iANT format)
	hitsort_files : path to the pairwise result file (could be a list, expl /path/to/dir/hitsort_*.Z
	outfile  : path to the outfile
	paired_ends_file : file with the identifiers of couple of transcripts known as paired ends (the ids of the couple have to be on the same line)
			exple : AL367179	AL367180
					MtRH15N17A1	MtRH15N17N1

=head1 AUTHORS
 
 Marion Verdenaud	marion.verdenaud@toulouse.inra.fr
 Sebastien Carrere  sebastien.carrere@toulouse.inra.fr
 Jerome Gouzy	    Jerome.Gouzy@toulouse.inra.fr

=cut



use strict;
use IO::File;

use ParamParser;
my $O_PARAM;

MAIN:
{
	$O_PARAM = New ParamParser("GETOPTLONG",("hitsort_files=s","outfile=s","paired_file=s","transcripts_file=s"));
	$O_PARAM->SetUsage(my $usage = sub {&Usage();});
	
	$O_PARAM->AssertFileExists('hitsort_files');
	$O_PARAM->AssertFileExists('paired_file');
	$O_PARAM->AssertFileExists('transcripts_file');
	
	my $file = $O_PARAM->Get('hitsort_files');
	my $outfile = $O_PARAM->Get('outfile');
	my $paired_end=$O_PARAM->Get('paired_file');
	my $transcripts_file= $O_PARAM->Get('transcripts_file');
	
	my $infile= new IO::File("gzip -cd $file |")||Carp::croak("can't open the file : gzip -cd $file |");
	my $outfile_h= new IO::File(">".$outfile)||Carp::croak("can't open the file : $outfile");
	my $paired_end_file = new IO::File($paired_end)||Carp::croak("can't open the paired end file");
	my $est_file = new IO::File($transcripts_file)||Carp::croak("can't open $transcripts_file");


	my %h_paired_ends;
	while (my $ligne=<$paired_end_file>)
	{
		my @a_paired = split ("\t",$ligne);
		my $pair1=$a_paired[0];
		my $pair2=$a_paired[1];
		
		chomp $pair1;
		chomp $pair2;
		
		$h_paired_ends{$pair1}=$pair2;
		$h_paired_ends{$pair2}=$pair1;
		
	}
	$paired_end_file->close;

	my %h_length;
	#the transcripts file is in iANT format
	my $est_file = new IO::File($transcripts_file)||Carp::croak("can't open $transcripts_file");
	while (my $line=<$est_file>)
	{
		if ($line =~ /acc=(\S+).*len=(\S+)/)
		{
			my $id=$1;
			my $length=$2;
			if (defined $h_paired_ends{$id})
			{
				$h_length{$id}=$length;
			}
		}
	}
	$est_file->close;

	my %h_treated;
 	while (my $ligne=<$infile>)
 	{
		
 		my @a_line=split ("\t",$ligne);
 		my $id1=$a_line[0];
 		chomp $id1;
 		
 		if ((! defined $h_treated{$id1}) && (defined $h_length{$id1}) && (defined $h_length{$h_paired_ends{$id1}}))
 		{
 			print $outfile_h "$id1\t".$h_length{$id1}."\t1\t100\t".$h_paired_ends{$id1}."\t".$h_length{$h_paired_ends{$id1}}."\t1\t100\t100\t6000\t3000\t+\n";
 		}
 		
 		$h_treated{$id1}="";
 		
 		my $id2=$a_line[4];
 		chomp $id2;
 
 		if ((! defined $h_treated{$id2}) && (defined $h_length{$id2}) && (defined $h_length{$h_paired_ends{$id2}}))
 		{
 			print $outfile_h "$id2\t".$h_length{$id2}."\t1\t100\t".$h_paired_ends{$id2}."\t".$h_length{$h_paired_ends{$id2}}."\t1\t100\t100\t6000\t3000\t+\n";
 		}
 		
 		$h_treated{$id2}="";
		
 	}
 	$infile->close;
	$outfile_h->close;


}

sub Usage
{
	print STDOUT <<END
Usage:

    $0 --hitsort_file --outfile --paired_end_file --transcripts_file

 Mandatory
    --hitsort_file      : path to the compressed output of the mgblast pairwise searches
    --outfile           : path to the outfile
    --paired_end_file   : path to the paired_end ids file use with PASA and TGICL 
    --transcripts_file  : path to the transcripts file use with TGICL (iANT format)
END
}